@@ -13,29 +13,50 @@ This module also defines ChatTabs, the parent class for all tabs
 revolving around chats.
 """
 
+import copy
 import logging
 import string
+import asyncio
 import time
+from math import ceil, log10
 from datetime import datetime
-from xml.etree import cElementTree as ET
-from typing import Any, Callable, Dict, List, Optional
-
-from slixmpp import JID, Message
+from xml.etree import ElementTree as ET
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Union,
+    TYPE_CHECKING,
+)
 
+from poezio import (
+    poopt,
+    timed_events,
+    xhtml,
+    windows
+)
 from poezio.core.structs import Command, Completion, Status
-from poezio import timed_events
-from poezio import windows
-from poezio import xhtml
 from poezio.common import safeJID
 from poezio.config import config
-from poezio.decorators import refresh_wrapper
+from poezio.decorators import command_args_parser, refresh_wrapper
 from poezio.logger import logger
 from poezio.text_buffer import TextBuffer
 from poezio.theming import get_theme, dump_tuple
-from poezio.decorators import command_args_parser
+from poezio.ui.funcs import truncate_nick
+from poezio.ui.consts import LONG_FORMAT_LENGTH
+from poezio.ui.types import BaseMessage, InfoMessage, Message
+
+from slixmpp import JID, InvalidJID, Message as SMessage
+
+if TYPE_CHECKING:
+    from _curses import _CursesWindow  # pylint: disable=E0611
 
 log = logging.getLogger(__name__)
 
+NS_MUC_USER = 'http://jabber.org/protocol/muc#user'
+
 # getters for tab colors (lambdas, so that they are dynamic)
 STATE_COLORS = {
     'disconnected': lambda: get_theme().COLOR_TAB_DISCONNECTED,
@@ -90,8 +111,11 @@ SHOW_NAME = {
 
 
 class Tab:
-    plugin_commands = {}
+    plugin_commands = {}  # type: Dict[str, Command]
     plugin_keys = {}  # type: Dict[str, Callable]
+    # Placeholder values, set on resize
+    height = 1
+    width = 1
 
     def __init__(self, core):
         self.core = core
@@ -172,7 +196,7 @@ class Tab:
             self._state = 'normal'
 
     @staticmethod
-    def resize(scr):
+    def resize(scr: '_CursesWindow'):
         Tab.height, Tab.width = scr.getmaxyx()
         windows.base_wins.TAB_WIN = scr
 
@@ -238,7 +262,7 @@ class Tab:
                     ['/%s' % (name) for name in sorted(self.commands)]
                 the_input.new_completion(words, 0)
                 # Do not try to cycle command completion if there was only
-                # one possibily. The next tab will complete the argument.
+                # one possibility. The next tab will complete the argument.
                 # Otherwise we would need to add a useless space before being
                 # able to complete the arguments.
                 hit_copy = set(the_input.hit_list)
@@ -455,16 +479,24 @@ class ChatTab(Tab):
     Also, ^M is already bound to on_enter
     And also, add the /say command
     """
-    plugin_commands = {}
-    plugin_keys = {}
+    plugin_commands = {}  # type: Dict[str, Command]
+    plugin_keys = {}  # type: Dict[str, Callable]
     message_type = 'chat'
 
-    def __init__(self, core, jid=''):
+    def __init__(self, core, jid: Union[JID, str]):
         Tab.__init__(self, core)
-        self.name = jid
-        self.text_win = None
+
+        if not isinstance(jid, JID):
+            jid = JID(jid)
+        assert jid.domain
+        self._jid = jid
+        #: Is the tab currently requesting MAM data?
+        self.query_status = False
+        self._name = jid.full  # type: Optional[str]
+        self.text_win = windows.TextWin()
         self.directed_presence = None
         self._text_buffer = TextBuffer()
+        self._text_buffer.add_window(self.text_win)
         self.chatstate = None  # can be "active", "composing", "paused", "gone", "inactive"
         # We keep a reference of the event that will set our chatstate to "paused", so that
         # we can delete it or change it if we need to
@@ -481,6 +513,12 @@ class ChatTab(Tab):
             self.command_say,
             usage='<message>',
             shortdesc='Send the message.')
+        self.register_command(
+            'scrollback',
+            self.command_scrollback,
+            usage="end home clear status goto <+|-linecount>|<linenum>|<timestamp>",
+            shortdesc='Scrollback to the given line number, message, or clear the buffer.')
+        self.commands['sb'] = self.commands['scrollback']
         self.register_command(
             'xhtml',
             self.command_xhtml,
@@ -498,56 +536,54 @@ class ChatTab(Tab):
         self.update_commands()
         self.update_keys()
 
-        # Get the logs
-        log_nb = config.get('load_log')
-        logs = self.load_logs(log_nb)
+    @property
+    def name(self) -> str:
+        if self._name is not None:
+            return self._name
+        return self._jid.full
 
-        if logs:
-            for message in logs:
-                self._text_buffer.add_message(**message)
+    @name.setter
+    def name(self, value: Union[JID, str]) -> None:
+        if isinstance(value, JID):
+            self.jid = value
+        elif isinstance(value, str):
+            try:
+                value = JID(value)
+                if value.domain:
+                    self._jid = value
+            except InvalidJID:
+                self._name = value
+        else:
+            raise TypeError("Name %r must be of type JID or str." % value)
+
+    @property
+    def jid(self) -> JID:
+        return copy.copy(self._jid)
+
+    @jid.setter
+    def jid(self, value: JID) -> None:
+        if not isinstance(value, JID):
+            raise TypeError("Jid %r must be of type JID." % value)
+        assert value.domain
+        self._jid = value
 
     @property
     def general_jid(self) -> JID:
-        return NotImplementedError
+        raise NotImplementedError
 
-    def load_logs(self, log_nb: int) -> Optional[List[Dict[str, Any]]]:
-        logs = logger.get_logs(safeJID(self.name).bare, log_nb)
-        return logs
-
-    def log_message(self,
-                    txt: str,
-                    nickname: str,
-                    time: Optional[datetime] = None,
-                    typ=1):
+    def log_message(self, message: BaseMessage, typ=1):
         """
         Log the messages in the archives.
         """
-        name = safeJID(self.name).bare
-        if not logger.log_message(name, nickname, txt, date=time, typ=typ):
+        name = self.jid.bare
+        if not isinstance(message, Message):
+            return
+        if not logger.log_message(name, message.nickname, message.txt, date=message.time, typ=typ):
             self.core.information('Unable to write in the log file', 'Error')
 
-    def add_message(self,
-                    txt,
-                    time=None,
-                    nickname=None,
-                    forced_user=None,
-                    nick_color=None,
-                    identifier=None,
-                    jid=None,
-                    history=None,
-                    typ=1,
-                    highlight=False):
-        self.log_message(txt, nickname, time=time, typ=typ)
-        self._text_buffer.add_message(
-            txt,
-            time=time,
-            nickname=nickname,
-            highlight=highlight,
-            nick_color=nick_color,
-            history=history,
-            user=forced_user,
-            identifier=identifier,
-            jid=jid)
+    def add_message(self, message: BaseMessage, typ=1):
+        self.log_message(message, typ=typ)
+        self._text_buffer.add_message(message)
 
     def modify_message(self,
                        txt,
@@ -556,11 +592,11 @@ class ChatTab(Tab):
                        user=None,
                        jid=None,
                        nickname=None):
-        self.log_message(txt, nickname, typ=1)
         message = self._text_buffer.modify_message(
-            txt, old_id, new_id, time=time, user=user, jid=jid)
+            txt, old_id, new_id, user=user, jid=jid)
         if message:
-            self.text_win.modify_message(old_id, message)
+            self.log_message(message, typ=1)
+            self.text_win.modify_message(message.identifier, message)
             self.core.refresh_window()
             return True
         return False
@@ -602,7 +638,7 @@ class ChatTab(Tab):
         if message:
             message.send()
 
-    def generate_xhtml_message(self, arg: str) -> Message:
+    def generate_xhtml_message(self, arg: str) -> SMessage:
         if not arg:
             return
         try:
@@ -621,7 +657,7 @@ class ChatTab(Tab):
         return msg
 
     def get_dest_jid(self) -> JID:
-        return self.name
+        return self.jid
 
     @refresh_wrapper.always
     def command_clear(self, ignored):
@@ -639,6 +675,8 @@ class ChatTab(Tab):
         """
         Send an empty chatstate message
         """
+        from poezio.tabs import PrivateTab
+
         if self.check_send_chat_state():
             if state in ('active', 'inactive',
                          'gone') and self.inactive and not always_send:
@@ -648,6 +686,10 @@ class ChatTab(Tab):
                 msg['type'] = self.message_type
                 msg['chat_state'] = state
                 self.chat_state = state
+                msg['no-store'] = True
+                if isinstance(self, PrivateTab):
+                    x = ET.Element('{%s}x' % NS_MUC_USER)
+                    msg.append(x)
                 msg.send()
                 return True
 
@@ -699,6 +741,15 @@ class ChatTab(Tab):
             self.core.remove_timed_event(self.timed_event_not_paused)
             self.timed_event_not_paused = None
 
+    def set_last_sent_message(self, msg, correct=False):
+        """Ensure last_sent_message is set with the correct attributes"""
+        if correct:
+            # XXX: Is the copy needed. Is the object passed here reused
+            # afterwards? Who knows.
+            msg = copy.copy(msg)
+            msg['id'] = self.last_sent_message['id']
+        self.last_sent_message = msg
+
     @command_args_parser.raw
     def command_correct(self, line):
         """
@@ -743,6 +794,128 @@ class ChatTab(Tab):
     def command_say(self, line, correct=False):
         pass
 
+    def goto_build_lines(self, new_date):
+        text_buffer = self._text_buffer
+        built_lines = []
+        message_count = 0
+        timestamp = config.get('show_timestamps')
+        nick_size = config.get('max_nick_length')
+        for message in text_buffer.messages:
+            # Build lines of a message
+            txt = message.txt
+            nick = truncate_nick(message.nickname, nick_size)
+            offset = 0
+            theme = get_theme()
+            if message.ack:
+                if message.ack > 0:
+                    offset += poopt.wcswidth(theme.CHAR_ACK_RECEIVED) + 1
+                else:
+                    offset += poopt.wcswidth(theme.CHAR_NACK) + 1
+            if nick:
+                offset += poopt.wcswidth(nick) + 2
+            if message.revisions > 0:
+                offset += ceil(log10(message.revisions + 1))
+            if message.me:
+                offset += 1
+            if timestamp:
+                if message.history:
+                    offset += 1 + LONG_FORMAT_LENGTH
+            lines = poopt.cut_text(txt, self.text_win.width - offset - 1)
+            for line in lines:
+                built_lines.append(line)
+            # Find the message with timestamp less than or equal to the queried
+            # timestamp and goto that location in the tab.
+            if message.time <= new_date:
+                message_count += 1
+                if len(self.text_win.built_lines) - self.text_win.height >= len(built_lines):
+                    self.text_win.pos = len(self.text_win.built_lines) - self.text_win.height - len(built_lines) + 1
+                else:
+                    self.text_win.pos = 0
+        if message_count == 0:
+            self.text_win.scroll_up(len(self.text_win.built_lines))
+        self.core.refresh_window()
+
+    @command_args_parser.quoted(0, 2)
+    def command_scrollback(self, args):
+        """
+        /sb clear
+        /sb home
+        /sb end
+        /sb goto <+|-linecount>|<linenum>|<timestamp>
+        The format of timestamp must be ‘[dd[.mm]-<days ago>] hh:mi[:ss]’
+        """
+        if args is None or len(args) == 0:
+            args = ['end']
+        if len(args) == 1:
+            if args[0] == 'end':
+                self.text_win.scroll_down(len(self.text_win.built_lines))
+                self.core.refresh_window()
+                return
+            elif args[0] == 'home':
+                self.text_win.scroll_up(len(self.text_win.built_lines))
+                self.core.refresh_window()
+                return
+            elif args[0] == 'clear':
+                self._text_buffer.messages = []
+                self.text_win.rebuild_everything(self._text_buffer)
+                self.core.refresh_window()
+                return
+            elif args[0] == 'status':
+                self.core.information('Total %s lines in this tab.' % len(self.text_win.built_lines), 'Info')
+                return
+        elif len(args) == 2 and args[0] == 'goto':
+            for fmt in ('%d %H:%M', '%d %H:%M:%S', '%d:%m %H:%M', '%d:%m %H:%M:%S', '%H:%M', '%H:%M:%S'):
+                try:
+                    new_date = datetime.strptime(args[1], fmt)
+                    if 'd' in fmt and 'm' in fmt:
+                        new_date = new_date.replace(year=datetime.now().year)
+                    elif 'd' in fmt:
+                        new_date = new_date.replace(year=datetime.now().year, month=datetime.now().month)
+                    else:
+                        new_date = new_date.replace(year=datetime.now().year, month=datetime.now().month, day=datetime.now().day)
+                except ValueError:
+                    pass
+            if args[1].startswith('-'):
+                # Check if the user is giving argument of type goto <-linecount> or goto [-<days ago>] hh:mi[:ss]
+                if ' ' in args[1]:
+                    new_args = args[1].split(' ')
+                    new_args[0] = new_args[0].strip('-')
+                    new_date = datetime.now()
+                    if new_args[0].isdigit():
+                        new_date = new_date.replace(day=new_date.day - int(new_args[0]))
+                    for fmt in ('%H:%M', '%H:%M:%S'):
+                        try:
+                            arg_date = datetime.strptime(new_args[1], fmt)
+                            new_date = new_date.replace(hour=arg_date.hour, minute=arg_date.minute, second=arg_date.second)
+                        except ValueError:
+                            pass
+                else:
+                    scroll_len = args[1].strip('-')
+                    if scroll_len.isdigit():
+                        self.text_win.scroll_down(int(scroll_len))
+                        self.core.refresh_window()
+                        return
+            elif args[1].startswith('+'):
+                scroll_len = args[1].strip('+')
+                if scroll_len.isdigit():
+                    self.text_win.scroll_up(int(scroll_len))
+                    self.core.refresh_window()
+                    return
+            # Check for the argument of type goto <linenum>
+            elif args[1].isdigit():
+                if len(self.text_win.built_lines) - self.text_win.height >= int(args[1]):
+                    self.text_win.pos = len(self.text_win.built_lines) - self.text_win.height - int(args[1])
+                    self.core.refresh_window()
+                    return
+                else:
+                    self.text_win.pos = 0
+                    self.core.refresh_window()
+                    return
+            elif args[1] == '0':
+                args = ['home']
+            # new_date is the timestamp for which the user has queried.
+            self.goto_build_lines(new_date)
+
     def on_line_up(self):
         return self.text_win.scroll_up(1)
 
@@ -750,6 +923,9 @@ class ChatTab(Tab):
         return self.text_win.scroll_down(1)
 
     def on_scroll_up(self):
+        if not self.query_status:
+            from poezio import mam
+            mam.schedule_scroll_up(tab=self)
         return self.text_win.scroll_up(self.text_win.height - 1)
 
     def on_scroll_down(self):
@@ -767,7 +943,7 @@ class ChatTab(Tab):
 
 
 class OneToOneTab(ChatTab):
-    def __init__(self, core, jid=''):
+    def __init__(self, core, jid):
         ChatTab.__init__(self, core, jid)
 
         self.__status = Status("", "")
@@ -798,7 +974,7 @@ class OneToOneTab(ChatTab):
             return
         self.__status = status
         hide_status_change = config.get_by_tabname('hide_status_change',
-                                                   safeJID(self.name).bare)
+                                                   self.jid.bare)
         now = datetime.now()
         dff = now - self.last_remote_message
         if hide_status_change > -1 and dff.total_seconds() > hide_status_change:
@@ -813,9 +989,12 @@ class OneToOneTab(ChatTab):
             msg += 'status: %s, ' % status.message
         if status.show in SHOW_NAME:
             msg += 'show: %s, ' % SHOW_NAME[status.show]
-        self.add_message(msg[:-2], typ=2)
+        self.add_message(
+            InfoMessage(txt=msg[:-2]),
+            typ=2,
+        )
 
-    def ack_message(self, msg_id, msg_jid):
+    def ack_message(self, msg_id: str, msg_jid: JID):
         """
         Ack a message
         """
@@ -824,9 +1003,9 @@ class OneToOneTab(ChatTab):
             self.text_win.modify_message(msg_id, new_msg)
             self.core.refresh_window()
 
-    def nack_message(self, error, msg_id, msg_jid):
+    def nack_message(self, error: str, msg_id: str, msg_jid: JID):
         """
-        Ack a message
+        Non-ack a message (e.g. timeout)
         """
         new_msg = self._text_buffer.nack_message(error, msg_id, msg_jid)
         if new_msg:
@@ -845,11 +1024,14 @@ class OneToOneTab(ChatTab):
             message.send()
             body = xhtml.xhtml_to_poezio_colors(xhtml_data, force=True)
             self._text_buffer.add_message(
-                body,
-                nickname=self.core.own_nick,
-                nick_color=get_theme().COLOR_OWN_NICK,
-                identifier=message['id'],
-                jid=self.core.xmpp.boundjid)
+                Message(
+                    body,
+                    nickname=self.core.own_nick,
+                    nick_color=get_theme().COLOR_OWN_NICK,
+                    identifier=message['id'],
+                    jid=self.core.xmpp.boundjid,
+                )
+            )
             self.refresh()
 
     def check_features(self):
@@ -863,7 +1045,7 @@ class OneToOneTab(ChatTab):
     @command_args_parser.raw
     def command_attention(self, message):
         """/attention [message]"""
-        if message is not '':
+        if message != '':
             self.command_say(message, attention=True)
         else:
             msg = self.core.xmpp.make_message(self.get_dest_jid())
