@@ -1,5 +1,5 @@
 """
-This plugin makes you have a random nick when joining a MUC.
+This plugin makes you have a random nick when joining a chatroom.
 
 Usage
 -----
@@ -11,9 +11,10 @@ automatically be changed to something random, for example: ::
 
 """
 
-from plugin import BasePlugin
+from poezio.plugin import BasePlugin
 from random import choice
 
+
 class Plugin(BasePlugin):
     def init(self):
         self.api.add_event_handler('joining_muc', self.change_nick_to_random)
@@ -25,11 +26,16 @@ class Plugin(BasePlugin):
             to.resource = gen_nick(3)
             presence["to"] = to
 
+
 s = ["i", "ou", "ou", "on", "a", "o", "u", "i"]
-c = ["b", "c", "d", "f", "g", "h", "j", "k", "m", "l", "n", "p", "r", "s", "t", "v", "z"]
+c = [
+    "b", "c", "d", "f", "g", "h", "j", "k", "m", "l", "n", "p", "r", "s", "t",
+    "v", "z"
+]
+
 
 def gen_nick(size):
     res = ''
-    for i in range(size):
+    for _ in range(size):
         res += '%s%s' % (choice(c), choice(s))
     return res
