@@ -30,29 +30,40 @@ from poezio.core.structs import Completion
 from poezio.decorators import command_args_parser
 from poezio import tabs
 
+
 class Plugin(BasePlugin):
     def init(self):
-        self.api.add_tab_command(tabs.MucTab, 'tell', self.command_tell,
-                usage='<nick> <message>',
-                help='Will tell <nick> of <message> when he next joins.',
-                short='Send a message when someone joins')
-        self.api.add_tab_command(tabs.MucTab, 'untell', self.command_untell,
-                usage='<nick>',
-                help='Remove the planned messages from /tell.',
-                short='Cancel a /tell message',
-                completion=self.completion_untell)
-        self.api.add_tab_command(tabs.MucTab, 'list_tell', self.command_list_tell,
-                usage='',
-                help='List currently queued messages')
+        self.api.add_tab_command(
+            tabs.MucTab,
+            'tell',
+            self.command_tell,
+            usage='<nick> <message>',
+            help='Will tell <nick> of <message> when he next joins.',
+            short='Send a message when someone joins')
+        self.api.add_tab_command(
+            tabs.MucTab,
+            'untell',
+            self.command_untell,
+            usage='<nick>',
+            help='Remove the planned messages from /tell.',
+            short='Cancel a /tell message',
+            completion=self.completion_untell)
+        self.api.add_tab_command(
+            tabs.MucTab,
+            'list_tell',
+            self.command_list_tell,
+            usage='',
+            help='List currently queued messages')
         self.api.add_event_handler('muc_join', self.on_join)
+        self.api.add_event_handler('muc_nickchange', self.on_join)
         # {tab -> {nick -> [messages]}
         self.tabs = {}
 
     def on_join(self, presence, tab):
-        if not tab in self.tabs:
+        if tab not in self.tabs:
             return
         nick = presence['from'].resource
-        if not nick in self.tabs[tab]:
+        if nick not in self.tabs[tab]:
             return
         for i in self.tabs[tab][nick]:
             tab.command_say("%s: %s" % (nick, i))
@@ -64,7 +75,7 @@ class Plugin(BasePlugin):
         if not self.tabs.get(tab):
             self.api.information('No message queued.', 'Info')
             return
-        build = ['Messages queued for %s:' % tab.name]
+        build = ['Messages queued for %s:' % tab.jid.bare]
         for nick, messages in self.tabs[tab].items():
             build.append(' for %s:' % nick)
             for message in messages:
@@ -79,9 +90,9 @@ class Plugin(BasePlugin):
             return
         nick, msg = args
         tab = self.api.current_tab()
-        if not tab in self.tabs:
+        if tab not in self.tabs:
             self.tabs[tab] = {}
-        if not nick in self.tabs[tab]:
+        if nick not in self.tabs[tab]:
             self.tabs[tab][nick] = []
         self.tabs[tab][nick].append(msg)
         self.api.information('Message for %s queued' % nick, 'Info')
@@ -89,17 +100,17 @@ class Plugin(BasePlugin):
     def command_untell(self, args):
         """/untell <nick>"""
         tab = self.api.current_tab()
-        if not tab in self.tabs:
+        if tab not in self.tabs:
             return
         nick = args
-        if not nick in self.tabs[tab]:
+        if nick not in self.tabs[tab]:
             return
         del self.tabs[tab][nick]
         self.api.information('Messages for %s unqueued' % nick, 'Info')
 
     def completion_untell(self, the_input):
         tab = self.api.current_tab()
-        if not tab in self.tabs:
+        if tab not in self.tabs:
             return Completion(the_input.auto_completion, [], '')
-        return Completion(the_input.auto_completion, list(self.tabs[tab]), '', quotify=False)
-
+        return Completion(
+            the_input.auto_completion, list(self.tabs[tab]), '', quotify=False)
