@@ -2,6 +2,7 @@
 Completions for the global commands
 """
 import logging
+from typing import List, Optional
 
 log = logging.getLogger(__name__)
 
@@ -9,6 +10,8 @@ import os
 from pathlib import Path
 from functools import reduce
 
+from slixmpp import JID
+
 from poezio import common
 from poezio import pep
 from poezio import tabs
@@ -41,6 +44,19 @@ class CompletionCore:
                 ' ',
                 quotify=False)
 
+    def roster_barejids(self, the_input):
+        """Complete roster bare jids"""
+        jids = sorted(
+            str(contact.bare_jid) for contact in roster.contacts.values()
+            if contact.pending_in
+        )
+        return Completion(the_input.new_completion, jids, 1, '', quotify=False)
+
+    def remove(self, the_input):
+        """Completion for /remove"""
+        jids = [jid for jid in roster.jids()]
+        return Completion(the_input.auto_completion, jids, '', quotify=False)
+
     def presence(self, the_input):
         """
         Completion of /presence
@@ -161,8 +177,8 @@ class CompletionCore:
         muc_serv_list = []
         for tab in self.core.get_tabs(
                 tabs.MucTab):  # TODO, also from an history
-            if tab.name not in muc_serv_list:
-                muc_serv_list.append(safeJID(tab.name).server)
+            if tab.jid.server not in muc_serv_list:
+                muc_serv_list.append(tab.jid.server)
         if muc_serv_list:
             return Completion(
                 the_input.new_completion, muc_serv_list, 1, quotify=False)
@@ -284,11 +300,24 @@ class CompletionCore:
             rooms = []
             for tab in self.core.get_tabs(tabs.MucTab):
                 if tab.joined:
-                    rooms.append(tab.name)
+                    rooms.append(tab.jid.bare)
             rooms.sort()
             return Completion(
                 the_input.new_completion, rooms, n, '', quotify=True)
 
+    def impromptu(self, the_input):
+        """Completion for /impromptu"""
+        n = the_input.get_argument_position(quoted=True)
+        onlines = []
+        offlines = []
+        for barejid in roster.jids():
+            if len(roster[barejid]):
+                onlines.append(barejid)
+            else:
+                offlines.append(barejid)
+        comp = sorted(onlines) + sorted(offlines)
+        return Completion(the_input.new_completion, comp, n, quotify=True)
+
     def activity(self, the_input):
         """Completion for /activity"""
         n = the_input.get_argument_position(quoted=True)
@@ -333,8 +362,7 @@ class CompletionCore:
         """Completion for /server_cycle"""
         serv_list = set()
         for tab in self.core.get_tabs(tabs.MucTab):
-            serv = safeJID(tab.name).server
-            serv_list.add(serv)
+            serv_list.add(tab.jid.server)
         return Completion(the_input.new_completion, sorted(serv_list), 1, ' ')
 
     def set(self, the_input):
@@ -449,3 +477,40 @@ class CompletionCore:
         muc_list = [tab.name for tab in self.core.get_tabs(tabs.MucTab)]
         muc_list.append('*')
         return Completion(the_input.new_completion, muc_list, 1, quotify=True)
+
+    def block(self, the_input) -> Optional[Completion]:
+        """
+        Completion for /block
+        """
+        if the_input.get_argument_position() == 1:
+
+            current_tab = self.core.tabs.current_tab
+            chattabs = (
+                tabs.ConversationTab,
+                tabs.StaticConversationTab,
+                tabs.DynamicConversationTab,
+            )
+            tabjid = []  # type: List[JID]
+            if isinstance(current_tab, chattabs):
+                tabjid = [current_tab.jid.bare]
+
+            jids = roster.jids()
+            jids += tabjid
+            return Completion(
+                the_input.new_completion, jids, 1, '', quotify=False)
+        return None
+
+    def unblock(self, the_input) -> Optional[Completion]:
+        """
+        Completion for /unblock
+        """
+
+        def on_result(iq):
+            if iq['type'] == 'error':
+                return None
+            l = sorted(str(item) for item in iq['blocklist']['items'])
+            return Completion(the_input.new_completion, l, 1, quotify=False)
+
+        if the_input.get_argument_position():
+            self.core.xmpp.plugin['xep_0191'].get_blocked(callback=on_result)
+        return None
