@@ -1,8 +1,9 @@
 """
 Once loaded, everything you will send will be IN CAPITAL LETTERS.
 """
-from plugin import BasePlugin
-import xhtml
+from poezio.plugin import BasePlugin
+from poezio import xhtml
+
 
 class Plugin(BasePlugin):
     def init(self):
