@@ -10,7 +10,7 @@ bookmarks, both local and remote.
 
 Poezio start scenario:
 
-- upon inital connection, poezio will disco#info the server
+- upon initial connection, poezio will disco#info the server
 - the available storage methods will be stored in the available_storage dict
     (either 'pep' or 'privatexml')
 - if only one is available, poezio will set the use_bookmarks_method config option
@@ -30,42 +30,69 @@ Adding a remote bookmark:
 
 import functools
 import logging
+from typing import Optional, List, Union
 
+from slixmpp import InvalidJID, JID
 from slixmpp.plugins.xep_0048 import Bookmarks, Conference, URL
-from slixmpp import JID
-from poezio.common import safeJID
 from poezio.config import config
 
 log = logging.getLogger(__name__)
 
 
-class Bookmark(object):
-
-    def __init__(self, jid, name=None, autojoin=False, nick=None, password=None, method='local'):
-        self.jid = jid
-        self.name = name or jid
+class Bookmark:
+    def __init__(self,
+                 jid: Union[JID, str],
+                 name: Optional[str] = None,
+                 autojoin=False,
+                 nick: Optional[str] = None,
+                 password: Optional[str] = None,
+                 method='local') -> None:
+        try:
+            if isinstance(jid, JID):
+                self._jid = jid
+            else:
+                self._jid = JID(jid)
+        except InvalidJID:
+            log.debug('Invalid JID %r provided for bookmark', jid)
+            raise
+        self.name = name or str(self.jid)
         self.autojoin = autojoin
         self.nick = nick
         self.password = password
         self._method = method
 
     @property
-    def method(self):
+    def jid(self) -> JID:
+        """Jid getter"""
+        return self._jid
+
+    @jid.setter
+    def jid(self, jid: JID) -> None:
+        try:
+            if isinstance(jid, JID):
+                self._jid = jid
+            else:
+                self._jid = JID(jid)
+        except InvalidJID:
+            log.debug('Invalid JID %r provided for bookmark', jid)
+            raise
+
+    @property
+    def method(self) -> str:
         return self._method
 
     @method.setter
-    def method(self, value):
+    def method(self, value: str):
         if value not in ('local', 'remote'):
             log.debug('Could not set bookmark storing method: %s', value)
             return
         self._method = value
 
-    def __repr__(self):
-        return '<%s%s|%s>' % (self.jid,
-                               ('/'+self.nick) if self.nick else '',
-                               self.method)
+    def __repr__(self) -> str:
+        return '<%s%s|%s>' % (self.jid, ('/' + self.nick)
+                              if self.nick else '', self.method)
 
-    def stanza(self):
+    def stanza(self) -> Conference:
         """
         Generate a <conference/> stanza from the instance
         """
@@ -79,9 +106,9 @@ class Bookmark(object):
             el['password'] = self.password
         return el
 
-    def local(self):
+    def local(self) -> str:
         """Generate a str for local storage"""
-        local = self.jid
+        local = str(self.jid)
         if self.nick:
             local += '/%s' % self.nick
         local += ':'
@@ -98,7 +125,8 @@ class Bookmark(object):
         """
         jid = el.get('jid')
         name = el.get('name')
-        autojoin = True if el.get('autojoin', 'false').lower() in ('true', '1') else False
+        autojoin = True if el.get('autojoin',
+                                  'false').lower() in ('true', '1') else False
         nick = None
         for n in el.iter('nick'):
             nick = n.text
@@ -121,10 +149,10 @@ class Bookmark(object):
         name = el['name']
         return Bookmark(jid, name, autojoin, nick, password, method='remote')
 
-class BookmarkList(object):
 
+class BookmarkList:
     def __init__(self):
-        self.bookmarks = []
+        self.bookmarks = []  # type: List[Bookmark]
         preferred = config.get('use_bookmarks_method').lower()
         if preferred not in ('pep', 'privatexml'):
             preferred = 'privatexml'
@@ -134,15 +162,16 @@ class BookmarkList(object):
             'pep': False,
         }
 
-    def __getitem__(self, key):
+    def __getitem__(self, key: Union[str, JID, int]) -> Optional[Bookmark]:
         if isinstance(key, (str, JID)):
             for i in self.bookmarks:
                 if key == i.jid:
                     return i
-        else:
+        elif isinstance(key, int):
             return self.bookmarks[key]
+        return None
 
-    def __in__(self, key):
+    def __contains__(self, key) -> bool:
         if isinstance(key, (str, JID)):
             for bookmark in self.bookmarks:
                 if bookmark.jid == key:
@@ -162,16 +191,16 @@ class BookmarkList(object):
     def __iter__(self):
         return iter(self.bookmarks)
 
-    def local(self):
+    def local(self) -> List[Bookmark]:
         return [bm for bm in self.bookmarks if bm.method == 'local']
 
-    def remote(self):
+    def remote(self) -> List[Bookmark]:
         return [bm for bm in self.bookmarks if bm.method == 'remote']
 
-    def set(self, new):
+    def set(self, new: List[Bookmark]):
         self.bookmarks = new
 
-    def append(self, bookmark):
+    def append(self, bookmark: Bookmark):
         bookmark_exists = self[bookmark.jid]
         if not bookmark_exists:
             self.bookmarks.append(bookmark)
@@ -179,7 +208,7 @@ class BookmarkList(object):
             self.bookmarks.remove(bookmark_exists)
             self.bookmarks.append(bookmark)
 
-    def set_bookmarks_method(self, value):
+    def set_bookmarks_method(self, value: str):
         if self.available_storage.get(value):
             self.preferred = value
             config.set_and_save('use_bookmarks_method', value)
@@ -191,17 +220,21 @@ class BookmarkList(object):
         method = 'xep_0049' if self.preferred == 'privatexml' else 'xep_0223'
 
         if method:
-            xmpp.plugin['xep_0048'].set_bookmarks(stanza_storage(self.bookmarks),
-                                                  method=method,
-                                                  callback=callback)
+            xmpp.plugin['xep_0048'].set_bookmarks(
+                stanza_storage(self.bookmarks),
+                method=method,
+                callback=callback)
+
     def save_local(self):
         """Save the local bookmarks."""
-        local = ''.join(bookmark.local() for bookmark in self if bookmark.method == 'local')
+        local = ''.join(bookmark.local() for bookmark in self
+                        if bookmark.method == 'local')
         config.set_and_save('rooms', local)
 
     def save(self, xmpp, core=None, callback=None):
         """Save all the bookmarks."""
         self.save_local()
+
         def _cb(iq):
             if callback:
                 callback(iq)
@@ -209,14 +242,17 @@ class BookmarkList(object):
                 core.information('Could not save remote bookmarks.', 'Error')
             elif core:
                 core.information('Bookmarks saved', 'Info')
+
         if config.get('use_remote_bookmarks'):
             self.save_remote(xmpp, _cb)
 
     def get_pep(self, xmpp, callback):
         """Add the remotely stored bookmarks via pep to the list."""
+
         def _cb(iq):
             if iq['type'] == 'result':
-                for conf in iq['pubsub']['items']['item']['bookmarks']['conferences']:
+                for conf in iq['pubsub']['items']['item']['bookmarks'][
+                        'conferences']:
                     if isinstance(conf, URL):
                         continue
                     b = Bookmark.parse(conf)
@@ -230,6 +266,7 @@ class BookmarkList(object):
         """
         Fetch the remote bookmarks stored via privatexml.
         """
+
         def _cb(iq):
             if iq['type'] == 'result':
                 for conf in iq['private']['bookmarks']['conferences']:
@@ -250,12 +287,15 @@ class BookmarkList(object):
         if force and not any(self.available_storage.values()):
             old_callback = callback
             method = 'pep' if self.preferred == 'pep' else 'privatexml'
+
             def new_callback(result):
                 if result['type'] != 'error':
                     self.available_storage[method] = True
                     old_callback(result)
                 else:
-                    information('No remote bookmark storage available', 'Warning')
+                    information('No remote bookmark storage available',
+                                'Warning')
+
             callback = new_callback
 
         if self.preferred == 'pep':
@@ -270,18 +310,29 @@ class BookmarkList(object):
             return
         rooms = rooms.split(':')
         for room in rooms:
-            jid = safeJID(room)
+            try:
+                jid = JID(room)
+            except InvalidJID:
+                continue
             if jid.bare == '':
                 continue
             if jid.resource != '':
                 nick = jid.resource
             else:
                 nick = None
-            passwd = config.get_by_tabname('password', jid.bare, fallback=False) or None
-            b = Bookmark(jid.bare, autojoin=True, nick=nick, password=passwd, method='local')
+            passwd = config.get_by_tabname(
+                'password', jid.bare, fallback=False) or None
+            b = Bookmark(
+                jid.bare,
+                jid.user,
+                autojoin=True,
+                nick=nick,
+                password=passwd,
+                method='local')
             self.append(b)
 
-def stanza_storage(bookmarks):
+
+def stanza_storage(bookmarks: BookmarkList) -> Bookmarks:
     """Generate a <storage/> stanza with the conference elements."""
     storage = Bookmarks()
     for b in (b for b in bookmarks if b.method == 'remote'):
