@@ -4,48 +4,32 @@
 #
 # Poezio is free software: you can redistribute it and/or modify
 # it under the terms of the zlib license. See the COPYING file.
-
 """
 Various useful functions.
 """
 
-from datetime import datetime, timedelta
-from slixmpp import JID, InvalidJID
-from poezio.poezio_shlex import shlex
+from datetime import (
+    datetime,
+    timedelta,
+    timezone,
+)
+from pathlib import Path
+from typing import Dict, List, Optional, Tuple, Union
 
-import base64
 import os
-import mimetypes
-import hashlib
 import subprocess
 import time
 import string
+import logging
+import itertools
+
+from slixmpp import JID, InvalidJID, Message
+from poezio.poezio_shlex import shlex
+
+log = logging.getLogger(__name__)
 
 
-def get_base64_from_file(path):
-    """
-    Convert the content of a file to base64
-
-    :param str path: The path of the file to convert.
-    :return: A tuple of (encoded data, mime type, sha1 hash) if
-        the file exists and does not exceeds the upper size limit of 16384.
-    :return: (None, None, error message) if it fails
-    :rtype: :py:class:`tuple`
-
-    """
-    if not os.path.isfile(path):
-        return (None, None, "File does not exist")
-    size = os.path.getsize(path)
-    if size > 16384:
-        return (None, None, "File is too big")
-    fdes = open(path, 'rb')
-    data = fdes.read()
-    encoded = base64.encodestring(data)
-    sha1 = hashlib.sha1(data).hexdigest()
-    mime_type = mimetypes.guess_type(path)[0]
-    return (encoded, mime_type, sha1)
-
-def _get_output_of_command(command):
+def _get_output_of_command(command: str) -> Optional[List[str]]:
     """
     Runs a command and returns its output.
 
@@ -54,11 +38,13 @@ def _get_output_of_command(command):
     :rtype: :py:class:`str`
     """
     try:
-        return subprocess.check_output(command.split()).decode('utf-8').split('\n')
+        return subprocess.check_output(
+            command.split()).decode('utf-8').split('\n')
     except subprocess.CalledProcessError:
         return None
 
-def _is_in_path(command, return_abs_path=False):
+
+def _is_in_path(command: str, return_abs_path=False) -> Union[bool, str]:
     """
     Check if *command* is in the $PATH or not.
 
@@ -69,43 +55,45 @@ def _is_in_path(command, return_abs_path=False):
         and *return_abs_path* is True, otherwise False.
 
     """
-    for directory in os.getenv('PATH').split(os.pathsep):
+    path = os.getenv('PATH', default='')
+    for directory in path.split(os.pathsep):
         try:
             if command in os.listdir(directory):
                 if return_abs_path:
                     return os.path.join(directory, command)
-                else:
-                    return True
+                return True
         except OSError:
             # If the user has non directories in his path
             pass
     return False
 
+
 DISTRO_INFO = {
-    'Arch Linux': '/etc/arch-release',
-    'Aurox Linux': '/etc/aurox-release',
-    'Conectiva Linux': '/etc/conectiva-release',
-    'CRUX': '/usr/bin/crux',
-    'Debian GNU/Linux': '/etc/debian_version',
-    'Fedora Linux': '/etc/fedora-release',
-    'Gentoo Linux': '/etc/gentoo-release',
-    'Linux from Scratch': '/etc/lfs-release',
-    'Mandrake Linux': '/etc/mandrake-release',
-    'Slackware Linux': '/etc/slackware-version',
-    'Solaris/Sparc': '/etc/release',
-    'Source Mage': '/etc/sourcemage_version',
-    'SUSE Linux': '/etc/SuSE-release',
-    'Sun JDS': '/etc/sun-release',
-    'PLD Linux': '/etc/pld-release',
-    'Yellow Dog Linux': '/etc/yellowdog-release',
+    'Arch Linux': Path('/etc/arch-release'),
+    'Aurox Linux': Path('/etc/aurox-release'),
+    'Conectiva Linux': Path('/etc/conectiva-release'),
+    'CRUX': Path('/usr/bin/crux'),
+    'Debian GNU/Linux': Path('/etc/debian_version'),
+    'Fedora Linux': Path('/etc/fedora-release'),
+    'Gentoo Linux': Path('/etc/gentoo-release'),
+    'Linux from Scratch': Path('/etc/lfs-release'),
+    'Mandrake Linux': Path('/etc/mandrake-release'),
+    'Slackware Linux': Path('/etc/slackware-version'),
+    'Solaris/Sparc': Path('/etc/release'),
+    'Source Mage': Path('/etc/sourcemage_version'),
+    'SUSE Linux': Path('/etc/SuSE-release'),
+    'Sun JDS': Path('/etc/sun-release'),
+    'PLD Linux': Path('/etc/pld-release'),
+    'Yellow Dog Linux': Path('/etc/yellowdog-release'),
     # many distros use the /etc/redhat-release for compatibility
     # so Redhat is the last
-    'Redhat Linux': '/etc/redhat-release'
+    'Redhat Linux': Path('/etc/redhat-release')
 }
 
-def get_os_info():
+
+def get_os_info() -> str:
     """
-    Returns a detailed and well formated string containing
+    Returns a detailed and well formatted string containing
     information about the operating system
 
     :rtype: str
@@ -116,10 +104,12 @@ def get_os_info():
         full_path_to_executable = _is_in_path(executable, return_abs_path=True)
         if full_path_to_executable:
             command = executable + params
-            process = subprocess.Popen([command], shell=True,
-                                       stdin=subprocess.PIPE,
-                                       stdout=subprocess.PIPE,
-                                       close_fds=True)
+            process = subprocess.Popen(
+                [command],
+                shell=True,
+                stdin=subprocess.PIPE,
+                stdout=subprocess.PIPE,
+                close_fds=True)
             process.wait()
             output = process.stdout.readline().decode('utf-8').strip()
             # some distros put n/a in places, so remove those
@@ -129,27 +119,30 @@ def get_os_info():
         # lsb_release executable not available, so parse files
         for distro_name in DISTRO_INFO:
             path_to_file = DISTRO_INFO[distro_name]
-            if os.path.exists(path_to_file):
-                if os.access(path_to_file, os.X_OK):
+            if path_to_file.exists():
+                if os.access(str(path_to_file), os.X_OK):
                     # the file is executable (f.e. CRUX)
                     # yes, then run it and get the first line of output.
-                    text = _get_output_of_command(path_to_file)[0]
+                    output = _get_output_of_command(str(path_to_file))
+                    if output:
+                        text = output[0]
+                    else:
+                        text = ''
                 else:
-                    fdes = open(path_to_file, encoding='utf-8')
-                    text = fdes.readline().strip() # get only first line
-                    fdes.close()
-                    if path_to_file.endswith('version'):
+                    with path_to_file.open(encoding='utf-8') as fdes:
+                        text = fdes.readline().strip()  # get only first line
+                    basename = path_to_file.name
+                    if basename.endswith('version'):
                         # sourcemage_version and slackware-version files
                         # have all the info we need (name and version of distro)
-                        if not os.path.basename(path_to_file).startswith(
-                                'sourcemage') or not\
-                                os.path.basename(path_to_file).startswith('slackware'):
+                        if not basename.startswith('sourcemage') or not\
+                                basename.startswith('slackware'):
                             text = distro_name + ' ' + text
-                    elif path_to_file.endswith('aurox-release') or \
-                            path_to_file.endswith('arch-release'):
+                    elif basename == 'aurox-release' or \
+                            basename == 'arch-release':
                         # file doesn't have version
                         text = distro_name
-                    elif path_to_file.endswith('lfs-release'):
+                    elif basename == 'lfs-release':
                         # file just has version
                         text = distro_name + ' ' + text
                 os_info = text.replace('\n', '')
@@ -158,12 +151,13 @@ def get_os_info():
         # our last chance, ask uname and strip it
         uname_output = _get_output_of_command('uname -sr')
         if uname_output is not None:
-            os_info = uname_output[0] # only first line
+            os_info = uname_output[0]  # only first line
             return os_info
     os_info = 'N/A'
     return os_info
 
-def _datetime_tuple(timestamp):
+
+def _datetime_tuple(timestamp: str) -> datetime:
     """
     Convert a timestamp using strptime and the format: %Y%m%dT%H:%M:%S.
 
@@ -189,12 +183,12 @@ def _datetime_tuple(timestamp):
     try:
         if tz_msg and tz_msg != 'Z':
             tz_mod = -1 if tz_msg[0] == '-' else 1
-            tz_msg = time.strptime(tz_msg[1:], '%H%M')
-            tz_msg = tz_msg.tm_hour * 3600 + tz_msg.tm_min * 60
-            tz_msg = timedelta(seconds=tz_mod * tz_msg)
-            ret -= tz_msg
+            tz_parsed = time.strptime(tz_msg[1:], '%H%M')
+            tz_seconds = tz_parsed.tm_hour * 3600 + tz_parsed.tm_min * 60
+            delta = timedelta(seconds=tz_mod * tz_seconds)
+            ret -= delta
     except ValueError:
-        pass # ignore if we got a badly-formatted offset
+        pass  # ignore if we got a badly-formatted offset
     # convert UTC to local time, with DST etc.
     if time.daylight and time.localtime().tm_isdst:
         tz = timedelta(seconds=-time.altzone)
@@ -203,7 +197,8 @@ def _datetime_tuple(timestamp):
     ret += tz
     return ret
 
-def get_utc_time(local_time=None):
+
+def get_utc_time(local_time: Optional[datetime] = None) -> datetime:
     """
     Get the current UTC time
 
@@ -225,7 +220,8 @@ def get_utc_time(local_time=None):
 
     return utc_time
 
-def get_local_time(utc_time):
+
+def get_local_time(utc_time: datetime) -> datetime:
     """
     Get the local time from an UTC time
     """
@@ -240,7 +236,8 @@ def get_local_time(utc_time):
 
     return local_time
 
-def find_delayed_tag(message):
+
+def find_delayed_tag(message: Message) -> Tuple[bool, Optional[datetime]]:
     """
     Check if a message is delayed or not.
 
@@ -251,6 +248,7 @@ def find_delayed_tag(message):
 
     find_delay = message.xml.find
     delay_tag = find_delay('{urn:xmpp:delay}delay')
+    date = None  # type: Optional[datetime]
     if delay_tag is not None:
         delayed = True
         date = _datetime_tuple(delay_tag.attrib['stamp'])
@@ -263,10 +261,10 @@ def find_delayed_tag(message):
             date = _datetime_tuple(delay_tag.attrib['stamp'])
         else:
             delayed = False
-            date = None
     return (delayed, date)
 
-def shell_split(st):
+
+def shell_split(st: str) -> List[str]:
     """
     Split a string correctly according to the quotes
     around the elements.
@@ -288,7 +286,8 @@ def shell_split(st):
         w = sh.get_token()
     return ret
 
-def find_argument(pos, text, quoted=True):
+
+def find_argument(pos: int, text: str, quoted=True) -> int:
     """
     Split an input into a list of arguments, return the number of the
     argument selected by pos.
@@ -305,10 +304,10 @@ def find_argument(pos, text, quoted=True):
     """
     if quoted:
         return _find_argument_quoted(pos, text)
-    else:
-        return _find_argument_unquoted(pos, text)
+    return _find_argument_unquoted(pos, text)
 
-def _find_argument_quoted(pos, text):
+
+def _find_argument_quoted(pos: int, text: str) -> int:
     """
     Get the number of the argument at position pos in
     a string with possibly quoted text.
@@ -324,7 +323,8 @@ def _find_argument_quoted(pos, text):
 
     return count + 1
 
-def _find_argument_unquoted(pos, text):
+
+def _find_argument_unquoted(pos: int, text: str) -> int:
     """
     Get the number of the argument at position pos in
     a string without interpreting quotes.
@@ -341,7 +341,8 @@ def _find_argument_unquoted(pos, text):
         argnum = i
     return argnum + 1
 
-def parse_str_to_secs(duration=''):
+
+def parse_str_to_secs(duration='') -> int:
     """
     Parse a string of with a number of d, h, m, s.
 
@@ -368,7 +369,8 @@ def parse_str_to_secs(duration=''):
         result += int(tmp)
     return result
 
-def parse_secs_to_str(duration=0):
+
+def parse_secs_to_str(duration=0) -> str:
     """
     Do the reverse operation of :py:func:`parse_str_to_secs`.
 
@@ -397,9 +399,10 @@ def parse_secs_to_str(duration=0):
         result = '0s'
     return result
 
-def format_tune_string(infos):
+
+def format_tune_string(infos: Dict[str, str]) -> str:
     """
-    Contruct a string from a dict created from an "User tune" event.
+    Construct a string from a dict created from an "User tune" event.
 
     :param dict infos: Tune information
     :return: The formatted string
@@ -424,17 +427,18 @@ def format_tune_string(infos):
     rating = infos.get('rating')
     if rating:
         elems.append('[ ' + rating + '/10 ]')
-    length = infos.get('length')
-    if length:
-        length = int(length)
+    length_str = infos.get('length')
+    if length_str:
+        length = int(length_str)
         secs = length % 60
         mins = length // 60
-        secs = str(secs).zfill(2)
-        mins = str(mins).zfill(2)
-        elems.append('[' + mins + ':' + secs + ']')
+        secs_str = str(secs).zfill(2)
+        mins_str = str(mins).zfill(2)
+        elems.append('[' + mins_str + ':' + secs_str + ']')
     return ' '.join(elems)
 
-def format_gaming_string(infos):
+
+def format_gaming_string(infos: Dict[str, str]) -> str:
     """
     Construct a string from a dict containing "user gaming" information.
     (for now, only use address and name)
@@ -452,7 +456,8 @@ def format_gaming_string(infos):
         return '%s on %s' % (name, server_address)
     return name
 
-def safeJID(*args, **kwargs):
+
+def safeJID(*args, **kwargs) -> JID:
     """
     Construct a :py:class:`slixmpp.JID` object from a string.
 
@@ -462,5 +467,39 @@ def safeJID(*args, **kwargs):
     try:
         return JID(*args, **kwargs)
     except InvalidJID:
+        log.debug(
+            'safeJID caught an invalidJID exception: %r, %r',
+            args, kwargs,
+            exc_info=True,
+        )
         return JID('')
 
+
+def unique_prefix_of(a: str, b: str) -> str:
+    """
+    Return the unique prefix of `a` with `b`.
+
+    Corner cases:
+
+    * If `a` and `b` share no prefix, the first letter of `a` is returned.
+    * If `a` and `b` are equal, `a` is returned.
+    * If `a` is a prefix of `b`, `a` is returned.
+    * If `b` is a prefix of `a`, `b` plus the first letter of `a` after the
+      common prefix is returned.
+    """
+    for i, (ca, cb) in enumerate(itertools.zip_longest(a, b)):
+        if ca != cb:
+            return a[:i+1]
+    # both are equal, return a
+    return a
+
+
+def to_utc(time: datetime) -> datetime:
+    """Convert a datetime-aware time zone into raw UTC"""
+    tzone = datetime.now().astimezone().tzinfo
+    if time.tzinfo is not None:  # Convert to UTC
+        time = time.astimezone(tz=timezone.utc)
+    else:  # Assume local tz, convert to URC
+        time = time.replace(tzinfo=tzone).astimezone(tz=timezone.utc)
+    # Return an offset-naive datetime
+    return time.replace(tzinfo=None)
