@@ -10,8 +10,6 @@ TODO: get http://bugs.python.org/issue1410680 fixed, one day, in order
 to remove our ugly custom I/O methods.
 """
 
-DEFSECTION = "Poezio"
-
 import logging.config
 import os
 import stat
@@ -19,9 +17,16 @@ import sys
 import pkg_resources
 
 from configparser import RawConfigParser, NoOptionError, NoSectionError
-from os import environ, makedirs, path, remove
+from pathlib import Path
 from shutil import copy2
+from typing import Callable, Dict, List, Optional, Union, Tuple
+
 from poezio.args import parse_args
+from poezio import xdg
+
+ConfigValue = Union[str, int, float, bool]
+
+DEFSECTION = "Poezio"
 
 DEFAULT_CONFIG = {
     'Poezio': {
@@ -32,7 +37,7 @@ DEFAULT_CONFIG = {
         'auto_reconnect': True,
         'autorejoin_delay': '5',
         'autorejoin': False,
-        'beep_on': 'highlight private invite',
+        'beep_on': 'highlight private invite disconnect',
         'ca_cert_path': '',
         'certificate': '',
         'certfile': '',
@@ -43,20 +48,24 @@ DEFAULT_CONFIG = {
         'custom_host': '',
         'custom_port': '',
         'default_nick': '',
+        'default_muc_service': '',
         'deterministic_nick_colors': True,
+        'device_id': '',
         'nick_color_aliases': True,
         'display_activity_notifications': False,
         'display_gaming_notifications': False,
         'display_mood_notifications': False,
         'display_tune_notifications': False,
         'display_user_color_in_join_part': True,
+        'enable_avatars': True,
         'enable_carbons': True,
+        'enable_css_parsing': True,
         'enable_user_activity': True,
         'enable_user_gaming': True,
         'enable_user_mood': True,
         'enable_user_nick': True,
         'enable_user_tune': True,
-        'enable_vertical_tab_list': False,
+        'enable_vertical_tab_list': True,
         'enable_xhtml_im': True,
         'enable_smacks': False,
         'eval_password': '',
@@ -73,13 +82,13 @@ DEFAULT_CONFIG = {
         'highlight_on': '',
         'ignore_certificate': False,
         'ignore_private': False,
+        'image_use_half_blocks': False,
         'information_buffer_popup_on': 'error roster warning help info',
         'information_buffer_type_filter': '',
         'jid': '',
         'keyfile': '',
         'lang': 'en',
         'lazy_resize': True,
-        'load_log': 10,
         'log_dir': '',
         'log_errors': True,
         'max_lines_in_memory': 2048,
@@ -101,7 +110,8 @@ DEFAULT_CONFIG = {
         'roster_show_offline': False,
         'roster_sort': 'jid:show',
         'save_status': True,
-        'self_ping_delay': 0,
+        'self_ping_interval': 0,
+        'self_ping_timeout': 60,
         'send_chat_states': True,
         'send_initial_presence': True,
         'send_os_info': True,
@@ -112,7 +122,7 @@ DEFAULT_CONFIG = {
         'show_composing_tabs': 'direct',
         'show_inactive_tabs': True,
         'show_jid_in_conversations': True,
-        'show_muc_jid': True,
+        'show_muc_jid': False,
         'show_roster_jids': True,
         'show_roster_subscriptions': '',
         'show_s2s_errors': True,
@@ -122,11 +132,13 @@ DEFAULT_CONFIG = {
         'show_useless_separator': True,
         'status': '',
         'status_message': '',
+        'synchronise_open_rooms': True,
         'theme': 'default',
         'themes_dir': '',
         'tmp_image_dir': '',
+        'unique_prefix_tab_names': False,
         'use_bookmarks_method': '',
-        'use_log': False,
+        'use_log': True,
         'use_remote_bookmarks': True,
         'user_list_sort': 'desc',
         'use_tab_nicks': True,
@@ -142,24 +154,25 @@ DEFAULT_CONFIG = {
         'folded_roster_groups': '',
         'info_win_height': 2
     },
-    'muc_colors': {
-    }
+    'muc_colors': {}
 }
 
+
 class Config(RawConfigParser):
     """
     load/save the config to a file
     """
-    def __init__(self, file_name, default=None):
+
+    def __init__(self, file_name: Path, default=None) -> None:
         RawConfigParser.__init__(self, None)
         # make the options case sensitive
-        self.optionxform = str
+        self.optionxform = lambda param: str(param)
         self.file_name = file_name
         self.read_file()
         self.default = default
 
     def read_file(self):
-        RawConfigParser.read(self, self.file_name, encoding='utf-8')
+        RawConfigParser.read(self, str(self.file_name), encoding='utf-8')
         # Check config integrity and fix it if it’s wrong
         # only when the object is the main config
         if self.__class__ is Config:
@@ -167,7 +180,10 @@ class Config(RawConfigParser):
                 if not self.has_section(section):
                     self.add_section(section)
 
-    def get(self, option, default=None, section=DEFSECTION):
+    def get(self,
+            option: str,
+            default: Optional[ConfigValue] = None,
+            section=DEFSECTION) -> ConfigValue:
         """
         get a value from the config but return
         a default value if it is not found
@@ -181,29 +197,36 @@ class Config(RawConfigParser):
                 default = ''
 
         try:
-            if type(default) == int:
-                res = self.getint(option, section)
-            elif type(default) == float:
-                res = self.getfloat(option, section)
-            elif type(default) == bool:
+            if isinstance(default, bool):
                 res = self.getboolean(option, section)
+            elif isinstance(default, int):
+                res = self.getint(option, section)
+            elif isinstance(default, float):
+                res = self.getfloat(option, section)
             else:
                 res = self.getstr(option, section)
         except (NoOptionError, NoSectionError, ValueError, AttributeError):
-            return default
+            return default if default is not None else ''
 
         if res is None:
             return default
         return res
 
-    def get_by_tabname(self, option, tabname,
-                       fallback=True, fallback_server=True, default=''):
+    def get_by_tabname(self,
+                       option,
+                       tabname: str,
+                       fallback=True,
+                       fallback_server=True,
+                       default=''):
         """
         Try to get the value for the option. First we look in
         a section named `tabname`, if the option is not present
         in the section, we search for the global option if fallback is
         True. And we return `default` as a fallback as a last resort.
         """
+        from slixmpp import JID
+        if isinstance(tabname, JID):
+            tabname = tabname.full
         if self.default and (not default) and fallback:
             default = self.default.get(DEFSECTION, {}).get(option, '')
         if tabname in self.sections():
@@ -230,7 +253,6 @@ class Config(RawConfigParser):
             return self.get(option, default)
         return default
 
-
     def __get(self, option, section=DEFSECTION, **kwargs):
         """
         facility for RawConfigParser.get
@@ -267,7 +289,8 @@ class Config(RawConfigParser):
         """
         return RawConfigParser.getboolean(self, section, option)
 
-    def write_in_file(self, section, option, value):
+    def write_in_file(self, section: str, option: str,
+                      value: ConfigValue) -> bool:
         """
         Our own way to save write the value in the file
         Just find the right section, and then find the
@@ -279,21 +302,21 @@ class Config(RawConfigParser):
         else:
             sections, result_lines = result
 
-        if not section in sections:
+        if section not in sections:
             result_lines.append('[%s]' % section)
             result_lines.append('%s = %s' % (option, value))
         else:
             begin, end = sections[section]
             pos = find_line(result_lines, begin, end, option)
 
-            if pos is -1:
+            if pos == -1:
                 result_lines.insert(end, '%s = %s' % (option, value))
             else:
                 result_lines[pos] = '%s = %s' % (option, value)
 
         return self._write_file(result_lines)
 
-    def remove_in_file(self, section, option):
+    def remove_in_file(self, section: str, option: str) -> bool:
         """
         Our own way to remove an option from the file.
         """
@@ -303,42 +326,44 @@ class Config(RawConfigParser):
         else:
             sections, result_lines = result
 
-        if not section in sections:
-            log.error('Tried to remove the option %s from a non-'
-                      'existing section (%s)', option, section)
+        if section not in sections:
+            log.error(
+                'Tried to remove the option %s from a non-'
+                'existing section (%s)', option, section)
             return True
         else:
             begin, end = sections[section]
             pos = find_line(result_lines, begin, end, option)
 
-            if pos is -1:
-                log.error('Tried to remove a non-existing option %s'
-                          ' from section %s', option, section)
+            if pos == -1:
+                log.error(
+                    'Tried to remove a non-existing option %s'
+                    ' from section %s', option, section)
                 return True
             else:
                 del result_lines[pos]
 
         return self._write_file(result_lines)
 
-    def _write_file(self, lines):
+    def _write_file(self, lines: List[str]) -> bool:
         """
         Write the config file, write to a temporary file
         before copying it to the final destination
         """
         try:
-            prefix, file = path.split(self.file_name)
-            filename = path.join(prefix, '.%s.tmp' % file)
-            fd = os.fdopen(
+            filename = self.file_name.parent / (
+                '.%s.tmp' % self.file_name.name)
+            with os.fdopen(
                     os.open(
-                        filename,
+                        str(filename),
                         os.O_WRONLY | os.O_CREAT,
-                        0o600),
-                    'w')
-            for line in lines:
-                fd.write('%s\n' % line)
-            fd.close()
-            copy2(filename, self.file_name)
-            remove(filename)
+                        0o600,
+                    ),
+                    'w',
+                    encoding='utf-8') as fd:
+                for line in lines:
+                    fd.write('%s\n' % line)
+            filename.replace(self.file_name)
         except:
             success = False
             log.error('Unable to save the config file.', exc_info=True)
@@ -346,7 +371,7 @@ class Config(RawConfigParser):
             success = True
         return success
 
-    def _parse_file(self):
+    def _parse_file(self) -> Optional[Tuple[Dict[str, List[int]], List[str]]]:
         """
         Parse the config file and return the list of sections with
         their start and end positions, and the lines in the file.
@@ -357,17 +382,19 @@ class Config(RawConfigParser):
         """
         if file_ok(self.file_name):
             try:
-                with open(self.file_name, 'r', encoding='utf-8') as df:
-                    lines_before = [line.strip() for line in df]
-            except:
-                log.error('Unable to read the config file %s',
-                        self.file_name,
-                        exc_info=True)
-                return tuple()
+                with self.file_name.open('r', encoding='utf-8') as df:
+                    lines_before = [line.strip()
+                                    for line in df]  # type: List[str]
+            except OSError:
+                log.error(
+                    'Unable to read the config file %s',
+                    self.file_name,
+                    exc_info=True)
+                return None
         else:
             lines_before = []
 
-        sections = {}
+        sections = {}  # type: Dict[str, List[int]]
         duplicate_section = False
         current_section = ''
         current_line = 0
@@ -393,7 +420,8 @@ class Config(RawConfigParser):
 
         return (sections, lines_before)
 
-    def set_and_save(self, option, value, section=DEFSECTION):
+    def set_and_save(self, option: str, value: ConfigValue,
+                     section=DEFSECTION) -> Tuple[str, str]:
         """
         set the value in the configuration then save it
         to the file
@@ -411,10 +439,10 @@ class Config(RawConfigParser):
                 elif current.lower() == "true":
                     value = "false"
                 else:
-                    return ('Could not toggle option: %s.'
-                            ' Current value is %s.' %
-                                (option, current or "empty"),
-                            'Warning')
+                    return (
+                        'Could not toggle option: %s.'
+                        ' Current value is %s.' % (option, current or "empty"),
+                        'Warning')
         if self.has_section(section):
             RawConfigParser.set(self, section, option, value)
         else:
@@ -422,9 +450,12 @@ class Config(RawConfigParser):
             RawConfigParser.set(self, section, option, value)
         if not self.write_in_file(section, option, value):
             return ('Unable to write in the config file', 'Error')
+        if isinstance(option, str) and 'password' in option and 'eval_password' not in option:
+            value = '********'
         return ("%s=%s" % (option, value), 'Info')
 
-    def remove_and_save(self, option, section=DEFSECTION):
+    def remove_and_save(self, option: str,
+                        section=DEFSECTION) -> Tuple[str, str]:
         """
         Remove an option and then save it the config file
         """
@@ -434,7 +465,7 @@ class Config(RawConfigParser):
             return ('Unable to save the config file', 'Error')
         return ('Option %s deleted' % option, 'Info')
 
-    def silent_set(self, option, value, section=DEFSECTION):
+    def silent_set(self, option: str, value: ConfigValue, section=DEFSECTION):
         """
         Set a value, save, and return True on success and False on failure
         """
@@ -445,7 +476,7 @@ class Config(RawConfigParser):
             RawConfigParser.set(self, section, option, value)
         return self.write_in_file(section, option, value)
 
-    def set(self, option, value, section=DEFSECTION):
+    def set(self, option: str, value: ConfigValue, section=DEFSECTION):
         """
         Set the value of an option temporarily
         """
@@ -454,11 +485,11 @@ class Config(RawConfigParser):
         except NoSectionError:
             pass
 
-    def to_dict(self):
+    def to_dict(self) -> Dict[str, Dict[str, ConfigValue]]:
         """
         Returns a dict of the form {section: {option: value, option: value}, …}
         """
-        res = {}
+        res = {}  # Dict[str, Dict[str, ConfigValue]]
         for section in self.sections():
             res[section] = {}
             for option in self.options(section):
@@ -466,7 +497,7 @@ class Config(RawConfigParser):
         return res
 
 
-def find_line(lines, start, end, option):
+def find_line(lines: List[str], start: int, end: int, option: str) -> int:
     """
     Get the number of the line containing the option in the
     relevant part of the config file.
@@ -475,52 +506,31 @@ def find_line(lines, start, end, option):
     """
     current = start
     for line in lines[start:end]:
-        if (line.startswith('%s ' % option) or
-                line.startswith('%s=' % option)):
+        if (line.startswith('%s ' % option)
+                or line.startswith('%s=' % option)):
             return current
         current += 1
     return -1
 
-def file_ok(filepath):
+
+def file_ok(filepath: Path) -> bool:
     """
     Returns True if the file exists and is readable and writeable,
     False otherwise.
     """
-    val = path.exists(filepath)
-    val &= os.access(filepath, os.R_OK | os.W_OK)
+    val = filepath.exists()
+    val &= os.access(str(filepath), os.R_OK | os.W_OK)
     return bool(val)
 
-def check_create_config_dir():
-    """
-    create the configuration directory if it doesn't exist
-    """
-    CONFIG_HOME = environ.get("XDG_CONFIG_HOME")
-    if not CONFIG_HOME:
-        CONFIG_HOME = path.join(environ.get('HOME'), '.config')
-    CONFIG_PATH = path.join(CONFIG_HOME, 'poezio')
 
-    try:
-        makedirs(CONFIG_PATH)
-    except OSError:
-        pass
-    return CONFIG_PATH
+def get_image_cache() -> Path:
+    if not config.get('extract_inline_images'):
+        return None
+    tmp_dir = config.get('tmp_image_dir')
+    if tmp_dir:
+        return Path(tmp_dir)
+    return xdg.CACHE_HOME / 'images'
 
-def check_create_cache_dir():
-    """
-    create the cache directory if it doesn't exist
-    also create the subdirectories
-    """
-    global CACHE_DIR
-    CACHE_HOME = environ.get("XDG_CACHE_HOME")
-    if not CACHE_HOME:
-        CACHE_HOME = path.join(environ.get('HOME'), '.cache')
-    CACHE_DIR = path.join(CACHE_HOME, 'poezio')
-
-    try:
-        makedirs(CACHE_DIR)
-        makedirs(path.join(CACHE_DIR, 'images'))
-    except OSError:
-        pass
 
 def check_config():
     """
@@ -530,7 +540,8 @@ def check_config():
     for option in DEFAULT_CONFIG['Poezio']:
         value = config.get(option)
         if value != DEFAULT_CONFIG['Poezio'][option]:
-            result['changed'].append((option, value, DEFAULT_CONFIG['Poezio'][option]))
+            result['changed'].append((option, value,
+                                      DEFAULT_CONFIG['Poezio'][option]))
         else:
             value = config.get(option, default='')
             upper = value.upper()
@@ -541,39 +552,51 @@ def check_config():
     result['changed'].sort(key=lambda x: x[0])
     result['missing'].sort()
     if result['changed']:
-        print('\033[1mOptions changed from the default configuration:\033[0m\n')
+        print(
+            '\033[1mOptions changed from the default configuration:\033[0m\n')
         for option, new_value, default in result['changed']:
-            print('    \033[1m%s\033[0m = \033[33m%s\033[0m (default: \033[32m%s\033[0m)' % (option, new_value, default))
+            print(
+                '    \033[1m%s\033[0m = \033[33m%s\033[0m (default: \033[32m%s\033[0m)'
+                % (option, new_value, default))
 
     if result['missing']:
         print('\n\033[1mMissing options:\033[0m (the defaults are used)\n')
         for option in result['missing']:
             print('    \033[31m%s\033[0m' % option)
 
-def run_cmdline_args(CONFIG_PATH):
+
+def run_cmdline_args():
     "Parse the command line arguments"
     global options
-    options = parse_args(CONFIG_PATH)
+    options = parse_args(xdg.CONFIG_HOME)
 
     # Copy a default file if none exists
-    if not path.isfile(options.filename):
-        default = path.join(path.dirname(__file__), '../data/default_config.cfg')
-        other = pkg_resources.resource_filename('poezio', 'default_config.cfg')
-        if path.isfile(default):
-            copy2(default, options.filename)
-        elif path.isfile(other):
-            copy2(other, options.filename)
+    if not options.filename.is_file():
+        try:
+            options.filename.parent.mkdir(parents=True, exist_ok=True)
+        except OSError as e:
+            sys.stderr.write(
+                'Poezio was unable to create the config directory: %s\n' % e)
+            sys.exit(1)
+        default = Path(__file__).parent / '..' / 'data' / 'default_config.cfg'
+        other = Path(
+            pkg_resources.resource_filename('poezio', 'default_config.cfg'))
+        if default.is_file():
+            copy2(str(default), str(options.filename))
+        elif other.is_file():
+            copy2(str(other), str(options.filename))
 
         # Inside the nixstore and possibly other distributions, the reference
         # file is readonly, so is the copy.
         # Make it writable by the user who just created it.
-        if os.path.exists(options.filename):
-            os.chmod(options.filename,
-                     os.stat(options.filename).st_mode | stat.S_IWUSR)
+        if options.filename.exists():
+            options.filename.chmod(options.filename.stat().st_mode
+                                   | stat.S_IWUSR)
 
         global firstrun
         firstrun = True
 
+
 def create_global_config():
     "Create the global config object, or crash"
     try:
@@ -586,50 +609,38 @@ def create_global_config():
         traceback.print_exc(limit=0)
         sys.exit(1)
 
-def check_create_log_dir():
-    "Create the poezio logging directory if it doesn’t exist"
-    global LOG_DIR
-    LOG_DIR = config.get('log_dir')
-
-    if not LOG_DIR:
-
-        data_dir = environ.get('XDG_DATA_HOME')
-        if not data_dir:
-            home = environ.get('HOME')
-            data_dir = path.join(home, '.local', 'share')
-
-        LOG_DIR = path.join(data_dir, 'poezio', 'logs')
-
-    LOG_DIR = path.expanduser(LOG_DIR)
-
-    try:
-        makedirs(LOG_DIR)
-    except:
-        pass
 
 def setup_logging():
     "Change the logging config according to the cmdline options and config"
+    global LOG_DIR
+    LOG_DIR = config.get('log_dir')
+    LOG_DIR = Path(LOG_DIR).expanduser() if LOG_DIR else xdg.DATA_HOME / 'logs'
     if config.get('log_errors'):
-        LOGGING_CONFIG['root']['handlers'].append('error')
-        LOGGING_CONFIG['handlers']['error'] = {
+        try:
+            LOG_DIR.mkdir(parents=True, exist_ok=True)
+        except OSError:
+            # We can’t really log any error here, because logging isn’t setup yet.
+            pass
+        else:
+            LOGGING_CONFIG['root']['handlers'].append('error')
+            LOGGING_CONFIG['handlers']['error'] = {
                 'level': 'ERROR',
                 'class': 'logging.FileHandler',
-                'filename': path.join(LOG_DIR, 'errors.log'),
+                'filename': str(LOG_DIR / 'errors.log'),
                 'formatter': 'simple',
             }
-        logging.disable(logging.WARNING)
+            logging.disable(logging.WARNING)
 
     if options.debug:
         LOGGING_CONFIG['root']['handlers'].append('debug')
         LOGGING_CONFIG['handlers']['debug'] = {
-                'level':'DEBUG',
-                'class':'logging.FileHandler',
-                'filename': options.debug,
-                'formatter': 'simple',
-            }
+            'level': 'DEBUG',
+            'class': 'logging.FileHandler',
+            'filename': options.debug,
+            'formatter': 'simple',
+        }
         logging.disable(logging.NOTSET)
 
-
     if LOGGING_CONFIG['root']['handlers']:
         logging.config.dictConfig(LOGGING_CONFIG)
     else:
@@ -639,6 +650,7 @@ def setup_logging():
     global log
     log = logging.getLogger(__name__)
 
+
 def post_logging_setup():
     # common imports slixmpp, which creates then its loggers, so
     # it needs to be after logger configuration
@@ -646,6 +658,7 @@ def post_logging_setup():
     global safeJID
     safeJID = JID
 
+
 LOGGING_CONFIG = {
     'version': 1,
     'disable_existing_loggers': True,
@@ -654,12 +667,11 @@ LOGGING_CONFIG = {
             'format': '%(asctime)s %(levelname)s:%(module)s:%(message)s'
         }
     },
-    'handlers': {
-    },
+    'handlers': {},
     'root': {
-            'handlers': [],
-            'propagate': True,
-            'level': 'DEBUG',
+        'handlers': [],
+        'propagate': True,
+        'level': 'DEBUG',
     }
 }
 
@@ -668,19 +680,16 @@ LOGGING_CONFIG = {
 firstrun = False
 
 # Global config object. Is setup in poezio.py
-config = None
+config = None  # type: Config
 
 # The logger object for this module
-log = None
+log = None  # type: Optional[logging.Logger]
 
 # The command-line options
 options = None
 
 # delayed import from common.py
-safeJID = None
+safeJID = None  # type: Optional[Callable]
 
 # the global log dir
-LOG_DIR = ''
-
-# the global cache dir
-CACHE_DIR = ''
+LOG_DIR = Path()
