@@ -3,6 +3,9 @@ Define the PluginConfig and Plugin classes, plus the SafetyMetaclass.
 These are used in the plugin system added in poezio 0.7.5
 (see plugin_manager.py)
 """
+
+from typing import Any, Dict, Set
+from asyncio import iscoroutinefunction
 from functools import partial
 from configparser import RawConfigParser
 from poezio.timed_events import TimedEvent, DelayedEvent
@@ -74,9 +77,12 @@ class SafetyMetaclass(type):
     @staticmethod
     def safe_func(f):
         def helper(*args, **kwargs):
+            passthrough = kwargs.pop('passthrough', False)
             try:
                 return f(*args, **kwargs)
             except:
+                if passthrough:
+                    raise
                 if inspect.stack()[1][1] == inspect.getfile(f):
                     raise
                 elif SafetyMetaclass.core:
@@ -84,9 +90,25 @@ class SafetyMetaclass(type):
                     SafetyMetaclass.core.information(traceback.format_exc(),
                                                      'Error')
                     return None
-
+        async def async_helper(*args, **kwargs):
+            passthrough = kwargs.pop('passthrough', False)
+            try:
+                return await f(*args, **kwargs)
+            except:
+                if passthrough:
+                    raise
+                if inspect.stack()[1][1] == inspect.getfile(f):
+                    raise
+                elif SafetyMetaclass.core:
+                    log.error('Error in a plugin', exc_info=True)
+                    SafetyMetaclass.core.information(traceback.format_exc(),
+                                                     'Error')
+                    return None
+        if iscoroutinefunction(f):
+            return async_helper
         return helper
 
+
     def __new__(meta, name, bases, class_dict):
         for k, v in class_dict.items():
             if inspect.isfunction(v):
@@ -379,19 +401,26 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
     Class that all plugins derive from.
     """
 
-    default_config = None
+    # Internal use only
+    _unloading = False
 
-    def __init__(self, plugin_api, core, plugins_conf_dir):
+    default_config = None
+    dependencies: Set[str] = set()
+    # This dict will get populated when the plugin is initialized
+    refs: Dict[str, Any] = {}
+
+    def __init__(self, name, plugin_api, core, plugins_conf_dir):
+        self.__name = name
         self.core = core
         # More hack; luckily we'll never have more than one core object
         SafetyMetaclass.core = core
-        conf = plugins_conf_dir / (self.__module__ + '.cfg')
+        conf = plugins_conf_dir / (self.__name + '.cfg')
         try:
             self.config = PluginConfig(
-                conf, self.__module__, default=self.default_config)
+                conf, self.__name, default=self.default_config)
         except Exception:
             log.debug('Error while creating the plugin config', exc_info=True)
-            self.config = PluginConfig(conf, self.__module__)
+            self.config = PluginConfig(conf, self.__name)
         self._api = plugin_api[self.name]
         self.init()
 
@@ -400,7 +429,7 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
         """
         Get the name (module name) of the plugin.
         """
-        return self.__module__
+        return self.__name
 
     @property
     def api(self):
@@ -501,12 +530,12 @@ class BasePlugin(object, metaclass=SafetyMetaclass):
         """
         return self.api.del_tab_command(tab_type, name)
 
-    def add_event_handler(self, event_name, handler, position=0):
+    def add_event_handler(self, event_name, handler, *args, **kwargs):
         """
         Add an event handler to the event event_name.
         An optional position in the event handler list can be provided.
         """
-        return self.api.add_event_handler(event_name, handler, position)
+        return self.api.add_event_handler(event_name, handler, *args, **kwargs)
 
     def del_event_handler(self, event_name, handler):
         """
