@@ -4,8 +4,6 @@
 #
 # Poezio is free software: you can redistribute it and/or modify
 # it under the terms of the zlib license. See the COPYING file.
-
-
 """
 Starting point of poezio. Launches both the Connection and Gui
 """
@@ -14,7 +12,6 @@ import sys
 import os
 import signal
 
-sys.path.append(os.path.dirname(os.path.abspath(__file__)))
 
 def test_curses():
     """
@@ -39,21 +36,58 @@ Poezio is currently unable to read your input or draw its interface properly,\
     return False
 
 
+def test_env():
+    """
+    Check if the environment has the right variables set
+    """
+    unset = set()
+    for i in ('HOME', 'PATH', 'TERM'):
+        if not os.environ.get(i):
+            unset.add(i)
+    if unset:
+        variables = ', '.join('$%s' % i for i in unset)
+        print('ERROR: %s not set' % variables)
+        return False
+    return True
+
+
+def test_unicode():
+    from . import poopt
+    try:
+        poopt.wcswidth('✔')
+    except UnicodeError:
+        print("""\
+ERROR: The current system is misconfigured for Unicode.
+
+Check your locale setup, especially the $LANG environment variable and \
+whether it matches a locale built on your system.  Also check that it is a \
+.UTF-8 locale, and not using some legacy encoding.
+
+Poezio is unable to display characters properly, so it will now exit.""")
+        return False
+    return True
+
+
 def main():
     """
     Entry point.
     """
+
+    if os.geteuid() == 0:
+        sys.stdout.write("Please do not run poezio as root.\n")
+        sys.exit(0)
+
     sys.stdout.write("\x1b]0;poezio\x07")
     sys.stdout.flush()
     from poezio import config
-    config_path = config.check_create_config_dir()
-    config.run_cmdline_args(config_path)
+    config.run_cmdline_args()
     config.create_global_config()
-    config.check_create_log_dir()
-    config.check_create_cache_dir()
     config.setup_logging()
     config.post_logging_setup()
 
+    import logging
+    logging.raiseExceptions = False
+
     from poezio.config import options
 
     if options.check_config:
@@ -72,11 +106,11 @@ def main():
     from poezio import roster
     roster.create_roster()
 
-    from poezio import core
+    from poezio.core.core import Core
 
-    signal.signal(signal.SIGINT, signal.SIG_IGN) # ignore ctrl-c
-    cocore = core.Core()
-    signal.signal(signal.SIGUSR1, cocore.sigusr_handler) # reload the config
+    signal.signal(signal.SIGINT, signal.SIG_IGN)  # ignore ctrl-c
+    cocore = Core()
+    signal.signal(signal.SIGUSR1, cocore.sigusr_handler)  # reload the config
     signal.signal(signal.SIGHUP, cocore.exit_from_signal)
     signal.signal(signal.SIGTERM, cocore.exit_from_signal)
     if options.debug:
@@ -84,6 +118,7 @@ def main():
     cocore.start()
 
     from slixmpp.exceptions import IqError, IqTimeout
+
     def swallow_iqerrors(loop, context):
         """Do not log unhandled iq errors and timeouts"""
         if not isinstance(context['exception'], (IqError, IqTimeout)):
