@@ -4,8 +4,6 @@
 #
 # Poezio is free software: you can redistribute it and/or modify
 # it under the terms of the zlib license. See the COPYING file.
-
-
 """
 Defines the Roster and RosterGroup classes
 """
@@ -22,22 +20,25 @@ from poezio.common import safeJID
 from slixmpp.exceptions import IqError, IqTimeout
 
 
-class Roster(object):
+class Roster:
     """
     The proxy class to get the roster from slixmpp.
     Caches Contact and RosterGroup objects.
     """
 
+    DEFAULT_FILTER = (lambda x, y: None, None)
+
     def __init__(self):
         """
         node: the RosterSingle from slixmpp
         """
         self.__node = None
-        self.contact_filter = None # A tuple(function, *args)
-                                    # function to filter contacts,
-                                    # on search, for example
-        self.folded_groups = set(config.get('folded_roster_groups',
-                                            section='var').split(':'))
+
+        # A tuple(function, *args) function to filter contacts
+        # on search, for example
+        self.contact_filter = self.DEFAULT_FILTER
+        self.folded_groups = set(
+            config.get('folded_roster_groups', section='var').split(':'))
         self.groups = {}
         self.contacts = {}
         self.length = 0
@@ -120,9 +121,8 @@ class Roster(object):
     def get_groups(self, sort=''):
         """Return a list of the RosterGroups"""
         group_list = sorted(
-                (group for group in self.groups.values() if group),
-                key=lambda x: x.name.lower() if x.name else ''
-            )
+            (group for group in self.groups.values() if group),
+            key=lambda x: x.name.lower() if x.name else '')
 
         for sorting in sort.split(':'):
             if sorting == 'reverse':
@@ -136,7 +136,8 @@ class Roster(object):
         """Return a group or create it if not present"""
         if name in self.groups:
             return self.groups[name]
-        self.groups[name] = RosterGroup(name, folded=name in self.folded_groups)
+        self.groups[name] = RosterGroup(
+            name, folded=name in self.folded_groups)
 
     def add(self, jid):
         """Subscribe to a jid"""
@@ -170,7 +171,7 @@ class Roster(object):
         contact_list = []
         for contact in self.get_contacts():
             if contact.bare_jid != self.jid:
-                if self.contact_filter:
+                if self.contact_filter is not self.DEFAULT_FILTER:
                     if self.contact_filter[0](contact, self.contact_filter[1]):
                         contact_list.append(contact)
                 else:
@@ -214,8 +215,9 @@ class Roster(object):
                 group.remove(contact)
 
         for group in contact.groups:
-            if not group in self.groups:
-                self.groups[group] = RosterGroup(group, folded=group in self.folded_groups)
+            if group not in self.groups:
+                self.groups[group] = RosterGroup(
+                    group, folded=group in self.folded_groups)
                 self.groups[group].add(contact)
 
     def __len__(self):
@@ -229,10 +231,10 @@ class Roster(object):
     def __repr__(self):
         ret = '== Roster:\nContacts:\n'
         for contact in self.contacts.values():
-            ret += '%s\n' % (contact,)
+            ret += '%s\n' % (contact, )
         ret += 'Groups\n'
         for group in self.groups:
-            ret += '%s\n' % (group,)
+            ret += '%s\n' % (group, )
         return ret + '\n'
 
     def export(self, path):
@@ -241,7 +243,10 @@ class Roster(object):
             return False
         try:
             f = open(path, 'w+', encoding='utf-8')
-            f.writelines([str(i) + "\n" for i in self.contacts if self[i] and (self[i].subscription == "both" or self[i].ask)])
+            f.writelines([
+                str(i) + "\n" for i in self.contacts
+                if self[i] and (self[i].subscription == "both" or self[i].ask)
+            ])
             f.close()
             return True
         except OSError:
@@ -256,18 +261,19 @@ class Roster(object):
         return True
 
 
-class RosterGroup(object):
+class RosterGroup:
     """
     A RosterGroup is a group containing contacts
     It can be Friends/Family etc, but also can be
     Online/Offline or whatever
     """
+
     def __init__(self, name, contacts=None, folded=False):
         if not contacts:
             contacts = []
         self.contacts = set(contacts)
         self.name = name if name is not None else ''
-        self.folded = folded    # if the group content is to be shown
+        self.folded = folded  # if the group content is to be shown
 
     def __iter__(self):
         """Iterate over the contacts"""
@@ -297,8 +303,13 @@ class RosterGroup(object):
 
     def get_contacts(self, contact_filter=None, sort=''):
         """Return the group contacts, filtered and sorted"""
-        contact_list = self.contacts.copy() if not contact_filter\
-            else [contact for contact in self.contacts.copy() if contact_filter[0](contact, contact_filter[1])]
+        if contact_filter is Roster.DEFAULT_FILTER or contact_filter is None:
+            contact_list = self.contacts.copy()
+        else:
+            contact_list = [
+                contact for contact in self.contacts.copy()
+                if contact_filter[0](contact, contact_filter[1])
+            ]
         contact_list = sorted(contact_list, key=SORTING_METHODS['name'])
 
         for sorting in sort.split(':'):
@@ -323,10 +334,12 @@ class RosterGroup(object):
         """Return the number of connected contacts"""
         return len([1 for contact in self.contacts if len(contact)])
 
+
 def create_roster():
     "Create the global roster object"
     global roster
     roster = Roster()
 
+
 # Shared roster object
 roster = None
