@@ -1,24 +1,25 @@
 """
-A generic tab that displays a serie of items in a scrollable, searchable,
+A generic tab that displays a series of items in a scrollable, searchable,
 sortable list.  It should be inherited, to actually provide methods that
 insert items in the list, and that lets the user interact with them.
 """
 
-import logging
-log = logging.getLogger(__name__)
-
 import curses
 import collections
+import logging
+from typing import Dict, Callable
 
 from poezio import windows
+from poezio.core.structs import Command
 from poezio.decorators import refresh_wrapper
-
 from poezio.tabs import Tab
 
+log = logging.getLogger(__name__)
+
 
 class ListTab(Tab):
-    plugin_commands = {}
-    plugin_keys = {}
+    plugin_commands = {}  # type: Dict[str, Command]
+    plugin_keys = {}  # type: Dict[str, Callable]
 
     def __init__(self, core, name, help_message, header_text, cols):
         """Parameters:
@@ -49,8 +50,7 @@ class ListTab(Tab):
         self.key_func['KEY_LEFT'] = self.list_header.sel_column_left
         self.key_func['KEY_RIGHT'] = self.list_header.sel_column_right
         self.key_func[' '] = self.sort_by
-        self.register_command('close', self.close,
-                shortdesc='Close this tab.')
+        self.register_command('close', self.close, shortdesc='Close this tab.')
         self.resize()
         self.update_keys()
         self.update_commands()
@@ -64,7 +64,6 @@ class ListTab(Tab):
         """
         raise NotImplementedError
 
-
     def refresh(self):
         if self.need_resize:
             self.resize()
@@ -90,26 +89,27 @@ class ListTab(Tab):
             info_win_height = self.core.information_win_size
             tab_win_height = Tab.tab_win_height()
 
-        self.info_header.resize(1, self.width,
-                                self.height - 2 - info_win_height
-                                    - tab_win_height,
-                                0)
+        self.info_header.resize(
+            1, self.width, self.height - 2 - info_win_height - tab_win_height,
+            0)
         column_size = self.get_columns_sizes()
         self.list_header.resize_columns(column_size)
         self.list_header.resize(1, self.width, 0, 0)
         self.listview.resize_columns(column_size)
-        self.listview.resize(self.height - 3 - info_win_height - tab_win_height,
-                             self.width, 1, 0)
-        self.input.resize(1, self.width, self.height-1, 0)
+        self.listview.resize(
+            self.height - 3 - info_win_height - tab_win_height, self.width, 1,
+            0)
+        self.input.resize(1, self.width, self.height - 1, 0)
 
     def on_slash(self):
         """
         '/' is pressed, activate the input
         """
         curses.curs_set(1)
-        self.input = windows.CommandInput("", self.reset_help_message, self.execute_slash_command)
-        self.input.resize(1, self.width, self.height-1, 0)
-        self.input.do_command("/") # we add the slash
+        self.input = windows.CommandInput("", self.reset_help_message,
+                                          self.execute_slash_command)
+        self.input.resize(1, self.width, self.height - 1, 0)
+        self.input.do_command("/")  # we add the slash
 
     def close(self, arg=None):
         self.core.close_tab(self)
@@ -118,7 +118,11 @@ class ListTab(Tab):
         """
         If there's an error (retrieving the values etc)
         """
-        self._error_message = 'Error: %(code)s - %(msg)s: %(body)s' % {'msg':msg, 'body':body, 'code':code}
+        self._error_message = 'Error: %(code)s - %(msg)s: %(body)s' % {
+            'msg': msg,
+            'body': body,
+            'code': code
+        }
         self.info_header.message = self._error_message
         self.info_header.refresh()
         curses.doupdate()
@@ -126,14 +130,12 @@ class ListTab(Tab):
     def sort_by(self):
         if self.list_header.get_order():
             self.listview.sort_by_column(
-                    col_name=self.list_header.get_sel_column(),
-                    asc=False)
+                col_name=self.list_header.get_sel_column(), asc=False)
             self.list_header.set_order(False)
             self.list_header.refresh()
         else:
             self.listview.sort_by_column(
-                    col_name=self.list_header.get_sel_column(),
-                    asc=True)
+                col_name=self.list_header.get_sel_column(), asc=True)
             self.list_header.set_order(True)
             self.list_header.refresh()
         self.core.doupdate()
@@ -144,7 +146,7 @@ class ListTab(Tab):
             return True
         curses.curs_set(0)
         self.input = self.default_help_message
-        self.input.resize(1, self.width, self.height-1, 0)
+        self.input.resize(1, self.width, self.height - 1, 0)
         return True
 
     def execute_slash_command(self, txt):
@@ -159,18 +161,20 @@ class ListTab(Tab):
 
     def on_input(self, key, raw):
         res = self.input.do_command(key, raw=raw)
-        if res and not isinstance(self.input, windows.Input):
-            return True
-        elif res:
-            return False
+        if res:
+            return not isinstance(self.input, windows.Input)
         if not raw and key in self.key_func:
             return self.key_func[key]()
 
     def on_info_win_size_changed(self):
-        if self.core.information_win_size >= self.height-3:
+        if self.core.information_win_size >= self.height - 3:
             return
-        self.info_header.resize(1, self.width, self.height-2-self.core.information_win_size - Tab.tab_win_height(), 0)
-        self.listview.resize(self.height-3-self.core.information_win_size - Tab.tab_win_height(), self.width, 1, 0)
+        self.info_header.resize(
+            1, self.width, self.height - 2 - self.core.information_win_size -
+            Tab.tab_win_height(), 0)
+        self.listview.resize(
+            self.height - 3 - self.core.information_win_size -
+            Tab.tab_win_height(), self.width, 1, 0)
 
     def on_lose_focus(self):
         self.state = 'normal'
@@ -197,5 +201,3 @@ class ListTab(Tab):
 
     def matching_names(self):
         return [(2, self.name)]
-
-
