@@ -6,16 +6,15 @@ rectangle shows the current contact info.
 This module also includes functions to match users in the roster.
 """
 import logging
-log = logging.getLogger(__name__)
-
 import base64
 import curses
 import difflib
 import os
 import ssl
-from os import getenv, path
 from functools import partial
+from os import getenv, path
 from pathlib import Path
+from typing import Dict, Callable
 
 from poezio import common
 from poezio import windows
@@ -25,18 +24,20 @@ from poezio.contact import Contact, Resource
 from poezio.decorators import refresh_wrapper
 from poezio.roster import RosterGroup, roster
 from poezio.theming import get_theme, dump_tuple
-from poezio.decorators import command_args_parser
-from poezio.core.structs import Completion
-
+from poezio.decorators import command_args_parser, deny_anonymous
+from poezio.core.structs import Command, Completion
 from poezio.tabs import Tab
+from poezio.ui.types import InfoMessage
+
+log = logging.getLogger(__name__)
 
 
 class RosterInfoTab(Tab):
     """
-    A tab, splitted in two, containing the roster and infos
+    A tab, split in two, containing the roster and infos
     """
-    plugin_commands = {}
-    plugin_keys = {}
+    plugin_commands = {}  # type: Dict[str, Command]
+    plugin_keys = {}  # type: Dict[str, Callable]
 
     def __init__(self, core):
         Tab.__init__(self, core)
@@ -71,96 +72,54 @@ class RosterInfoTab(Tab):
             self.key_func["s"] = self.start_search
             self.key_func["S"] = self.start_search_slow
             self.key_func["n"] = self.change_contact_name
-            self.register_command(
-                'deny',
-                self.command_deny,
-                usage='[jid]',
-                desc='Deny your presence to the provided JID (or the '
-                'selected contact in your roster), who is asking'
-                'you to be in his/here roster.',
-                shortdesc='Deny a user your presence.',
-                completion=self.completion_deny)
-            self.register_command(
-                'accept',
-                self.command_accept,
-                usage='[jid]',
-                desc='Allow the provided JID (or the selected contact '
-                'in your roster), to see your presence.',
-                shortdesc='Allow a user your presence.',
-                completion=self.completion_deny)
-            self.register_command(
-                'add',
-                self.command_add,
-                usage='<jid>',
-                desc='Add the specified JID to your roster, ask them to'
-                ' allow you to see his presence, and allow them to'
-                ' see your presence.',
-                shortdesc='Add a user to your roster.')
-            self.register_command(
-                'name',
-                self.command_name,
-                usage='<jid> [name]',
-                shortdesc='Set the given JID\'s name.',
-                completion=self.completion_name)
-            self.register_command(
-                'groupadd',
-                self.command_groupadd,
-                usage='[<jid> <group>]|<group>',
-                desc='Add the given JID or selected line to the given group.',
-                shortdesc='Add a user to a group',
-                completion=self.completion_groupadd)
-            self.register_command(
-                'groupmove',
-                self.command_groupmove,
-                usage='<jid> <old group> <new group>',
-                desc='Move the given JID from the old group to the new group.',
-                shortdesc='Move a user to another group.',
-                completion=self.completion_groupmove)
-            self.register_command(
-                'groupremove',
-                self.command_groupremove,
-                usage='<jid> <group>',
-                desc='Remove the given JID from the given group.',
-                shortdesc='Remove a user from a group.',
-                completion=self.completion_groupremove)
-            self.register_command(
-                'remove',
-                self.command_remove,
-                usage='[jid]',
-                desc='Remove the specified JID from your roster. This '
-                'will unsubscribe you from its presence, cancel '
-                'its subscription to yours, and remove the item '
-                'from your roster.',
-                shortdesc='Remove a user from your roster.',
-                completion=self.completion_remove)
-            self.register_command(
-                'export',
-                self.command_export,
-                usage='[/path/to/file]',
-                desc='Export your contacts into /path/to/file if '
-                'specified, or $HOME/poezio_contacts if not.',
-                shortdesc='Export your roster to a file.',
-                completion=partial(self.completion_file, 1))
-            self.register_command(
-                'import',
-                self.command_import,
-                usage='[/path/to/file]',
-                desc='Import your contacts from /path/to/file if '
-                'specified, or $HOME/poezio_contacts if not.',
-                shortdesc='Import your roster from a file.',
-                completion=partial(self.completion_file, 1))
-            self.register_command(
-                'password',
-                self.command_password,
-                usage='<password>',
-                shortdesc='Change your password')
-
         self.register_command(
-            'reconnect',
-            self.command_reconnect,
-            desc='Disconnect from the remote server if you are '
-            'currently connected and then connect to it again.',
-            shortdesc='Disconnect and reconnect to the server.')
+            'name',
+            self.command_name,
+            usage='<jid> [name]',
+            shortdesc='Set the given JID\'s name.',
+            completion=self.completion_name)
+        self.register_command(
+            'groupadd',
+            self.command_groupadd,
+            usage='[<jid> <group>]|<group>',
+            desc='Add the given JID or selected line to the given group.',
+            shortdesc='Add a user to a group',
+            completion=self.completion_groupadd)
+        self.register_command(
+            'groupmove',
+            self.command_groupmove,
+            usage='<jid> <old group> <new group>',
+            desc='Move the given JID from the old group to the new group.',
+            shortdesc='Move a user to another group.',
+            completion=self.completion_groupmove)
+        self.register_command(
+            'groupremove',
+            self.command_groupremove,
+            usage='<jid> <group>',
+            desc='Remove the given JID from the given group.',
+            shortdesc='Remove a user from a group.',
+            completion=self.completion_groupremove)
+        self.register_command(
+            'export',
+            self.command_export,
+            usage='[/path/to/file]',
+            desc='Export your contacts into /path/to/file if '
+            'specified, or $HOME/poezio_contacts if not.',
+            shortdesc='Export your roster to a file.',
+            completion=partial(self.completion_file, 1))
+        self.register_command(
+            'import',
+            self.command_import,
+            usage='[/path/to/file]',
+            desc='Import your contacts from /path/to/file if '
+            'specified, or $HOME/poezio_contacts if not.',
+            shortdesc='Import your roster from a file.',
+            completion=partial(self.completion_file, 1))
+        self.register_command(
+            'password',
+            self.command_password,
+            usage='<password>',
+            shortdesc='Change your password')
         self.register_command(
             'disconnect',
             self.command_disconnect,
@@ -182,18 +141,6 @@ class RosterInfoTab(Tab):
 
     def check_blocking(self, features):
         if 'urn:xmpp:blocking' in features and not self.core.xmpp.anon:
-            self.register_command(
-                'block',
-                self.command_block,
-                usage='[jid]',
-                shortdesc='Prevent a JID from talking to you.',
-                completion=self.completion_block)
-            self.register_command(
-                'unblock',
-                self.command_unblock,
-                usage='[jid]',
-                shortdesc='Allow a JID to talk to you.',
-                completion=self.completion_unblock)
             self.register_command(
                 'list_blocks',
                 self.command_list_blocks,
@@ -426,79 +373,11 @@ class RosterInfoTab(Tab):
         if not tab:
             log.debug('Received message from nonexistent tab: %s',
                       message['from'])
-        message = '\x19%(info_col)s}Cannot send message to %(jid)s: contact blocked' % {
+        message = 'Cannot send message to %(jid)s: contact blocked' % {
             'info_col': dump_tuple(get_theme().COLOR_INFORMATION_TEXT),
             'jid': message['from'],
         }
-        tab.add_message(message)
-
-    @command_args_parser.quoted(0, 1)
-    def command_block(self, args):
-        """
-        /block [jid]
-        """
-        item = self.roster_win.selected_row
-        if args:
-            jid = safeJID(args[0])
-        elif isinstance(item, Contact):
-            jid = item.bare_jid
-        elif isinstance(item, Resource):
-            jid = item.jid.bare
-
-        def callback(iq):
-            if iq['type'] == 'error':
-                return self.core.information('Could not block %s.' % jid,
-                                             'Error')
-            elif iq['type'] == 'result':
-                return self.core.information('Blocked %s.' % jid, 'Info')
-
-        self.core.xmpp.plugin['xep_0191'].block(jid, callback=callback)
-
-    def completion_block(self, the_input):
-        """
-        Completion for /block
-        """
-        if the_input.get_argument_position() == 1:
-            jids = roster.jids()
-            return Completion(
-                the_input.new_completion, jids, 1, '', quotify=False)
-
-    @command_args_parser.quoted(0, 1)
-    def command_unblock(self, args):
-        """
-        /unblock [jid]
-        """
-
-        def callback(iq):
-            if iq['type'] == 'error':
-                return self.core.information('Could not unblock the contact.',
-                                             'Error')
-            elif iq['type'] == 'result':
-                return self.core.information('Contact unblocked.', 'Info')
-
-        item = self.roster_win.selected_row
-        if args:
-            jid = safeJID(args[0])
-        elif isinstance(item, Contact):
-            jid = item.bare_jid
-        elif isinstance(item, Resource):
-            jid = item.jid.bare
-        self.core.xmpp.plugin['xep_0191'].unblock(jid, callback=callback)
-
-    def completion_unblock(self, the_input):
-        """
-        Completion for /unblock
-        """
-
-        def on_result(iq):
-            if iq['type'] == 'error':
-                return
-            l = sorted(str(item) for item in iq['blocklist']['items'])
-            return Completion(the_input.new_completion, l, 1, quotify=False)
-
-        if the_input.get_argument_position():
-            self.core.xmpp.plugin['xep_0191'].get_blocked(callback=on_result)
-        return True
+        tab.add_message(InfoMessage(message), typ=0)
 
     @command_args_parser.ignored
     def command_list_blocks(self):
@@ -521,16 +400,6 @@ class RosterInfoTab(Tab):
 
         self.core.xmpp.plugin['xep_0191'].get_blocked(callback=callback)
 
-    @command_args_parser.ignored
-    def command_reconnect(self):
-        """
-        /reconnect
-        """
-        if self.core.xmpp.is_connected():
-            self.core.disconnect(reconnect=True)
-        else:
-            self.core.xmpp.connect()
-
     @command_args_parser.ignored
     def command_disconnect(self):
         """
@@ -611,7 +480,7 @@ class RosterInfoTab(Tab):
         n = the_input.get_argument_position()
         if n == complete_number:
             if args[n - 1] == '' or len(args) < n + 1:
-                home = os.getenv('HOME') or '/'
+                home = os.getenv('HOME', default='/')
                 return Completion(
                     the_input.new_completion, [home, '/tmp'], n, quotify=True)
             path_ = Path(args[n])
@@ -652,6 +521,7 @@ class RosterInfoTab(Tab):
             self.core.information_buffer)
         self.refresh()
 
+    @deny_anonymous
     @command_args_parser.quoted(1)
     def command_password(self, args):
         """
@@ -670,51 +540,8 @@ class RosterInfoTab(Tab):
         self.core.xmpp.plugin['xep_0077'].change_password(
             args[0], callback=callback)
 
-    @command_args_parser.quoted(0, 1)
-    def command_deny(self, args):
-        """
-        /deny [jid]
-        Denies a JID from our roster
-        """
-        if not args:
-            item = self.roster_win.selected_row
-            if isinstance(item, Contact):
-                jid = item.bare_jid
-            else:
-                self.core.information('No subscription to deny', 'Warning')
-                return
-        else:
-            jid = safeJID(args[0]).bare
-            if jid not in [jid for jid in roster.jids()]:
-                self.core.information('No subscription to deny', 'Warning')
-                return
-
-        contact = roster[jid]
-        if contact:
-            contact.unauthorize()
-            self.core.information('Subscription to %s was revoked' % jid,
-                                  'Roster')
-
-    @command_args_parser.quoted(1)
-    def command_add(self, args):
-        """
-        Add the specified JID to the roster, and automatically
-        accept the reverse subscription
-        """
-        if args is None:
-            self.core.information('No JID specified', 'Error')
-            return
-        jid = safeJID(safeJID(args[0]).bare)
-        if not str(jid):
-            self.core.information(
-                'The provided JID (%s) is not valid' % (args[0], ), 'Error')
-            return
-        if jid in roster and roster[jid].subscription in ('to', 'both'):
-            return self.core.information('Already subscribed.', 'Roster')
-        roster.add(jid)
-        roster.modified()
-        self.core.information('%s was added to the roster' % jid, 'Roster')
 
+    @deny_anonymous
     @command_args_parser.quoted(1, 1)
     def command_name(self, args):
         """
@@ -747,6 +574,7 @@ class RosterInfoTab(Tab):
             subscription=subscription,
             callback=callback)
 
+    @deny_anonymous
     @command_args_parser.quoted(1, 1)
     def command_groupadd(self, args):
         """
@@ -801,6 +629,7 @@ class RosterInfoTab(Tab):
             subscription=subscription,
             callback=callback)
 
+    @deny_anonymous
     @command_args_parser.quoted(3)
     def command_groupmove(self, args):
         """
@@ -860,6 +689,7 @@ class RosterInfoTab(Tab):
             subscription=subscription,
             callback=callback)
 
+    @deny_anonymous
     @command_args_parser.quoted(2)
     def command_groupremove(self, args):
         """
@@ -905,24 +735,7 @@ class RosterInfoTab(Tab):
             subscription=subscription,
             callback=callback)
 
-    @command_args_parser.quoted(0, 1)
-    def command_remove(self, args):
-        """
-        Remove the specified JID from the roster. i.e.: unsubscribe
-        from its presence, and cancel its subscription to our.
-        """
-        if args:
-            jid = safeJID(args[0]).bare
-        else:
-            item = self.roster_win.selected_row
-            if isinstance(item, Contact):
-                jid = item.bare_jid
-            else:
-                self.core.information('No roster item to remove', 'Error')
-                return
-        roster.remove(jid)
-        del roster[jid]
-
+    @deny_anonymous
     @command_args_parser.quoted(0, 1)
     def command_import(self, args):
         """
@@ -948,9 +761,10 @@ class RosterInfoTab(Tab):
             log.error('Unable to correct a message', exc_info=True)
             return
         for jid in lines:
-            self.command_add(jid.lstrip('\n'))
+            self.core.command.command_add(jid.lstrip('\n'))
         self.core.information('Contacts imported from %s' % filepath, 'Info')
 
+    @deny_anonymous
     @command_args_parser.quoted(0, 1)
     def command_export(self, args):
         """
@@ -1045,49 +859,6 @@ class RosterInfoTab(Tab):
                 the_input.new_completion, groups, n, '', quotify=True)
         return False
 
-    def completion_deny(self, the_input):
-        """
-        Complete the first argument from the list of the
-        contact with ask=='subscribe'
-        """
-        jids = sorted(
-            str(contact.bare_jid) for contact in roster.contacts.values()
-            if contact.pending_in)
-        return Completion(the_input.new_completion, jids, 1, '', quotify=False)
-
-    @command_args_parser.quoted(0, 1)
-    def command_accept(self, args):
-        """
-        Accept a JID from in roster. Authorize it AND subscribe to it
-        """
-        if not args:
-            item = self.roster_win.selected_row
-            if isinstance(item, Contact):
-                jid = item.bare_jid
-            else:
-                self.core.information('No subscription to accept', 'Warning')
-                return
-        else:
-            jid = safeJID(args[0]).bare
-        nodepart = safeJID(jid).user
-        jid = safeJID(jid)
-        # crappy transports putting resources inside the node part
-        if '\\2f' in nodepart:
-            jid.user = nodepart.split('\\2f')[0]
-        contact = roster[jid]
-        if contact is None:
-            return
-        contact.pending_in = False
-        roster.modified()
-        self.core.xmpp.send_presence(pto=jid, ptype='subscribed')
-        self.core.xmpp.client_roster.send_last_presence()
-        if contact.subscription in ('from',
-                                    'none') and not contact.pending_out:
-            self.core.xmpp.send_presence(
-                pto=jid, ptype='subscribe', pnick=self.core.own_nick)
-
-        self.core.information('%s is now authorized' % jid, 'Roster')
-
     def refresh(self):
         if self.need_resize:
             self.resize()
@@ -1114,10 +885,8 @@ class RosterInfoTab(Tab):
         if key == '^M':
             selected_row = self.roster_win.get_selected_row()
         res = self.input.do_command(key, raw=raw)
-        if res and not isinstance(self.input, windows.Input):
-            return True
-        elif res:
-            return False
+        if res:
+            return not isinstance(self.input, windows.Input)
         if key == '^M':
             self.core.on_roster_enter_key(selected_row)
             return selected_row
