@@ -7,7 +7,7 @@ the text window, the roster window, etc.
 A Tab (see the poezio.tabs module) is composed of multiple Windows
 """
 
-TAB_WIN = None
+TAB_WIN = None  # type: _CursesWindow
 
 import logging
 log = logging.getLogger(__name__)
@@ -15,30 +15,36 @@ log = logging.getLogger(__name__)
 import curses
 import string
 
+from contextlib import contextmanager
+from typing import Optional, Tuple, TYPE_CHECKING
+
 from poezio.theming import to_curses_attr, read_tuple
 
-FORMAT_CHAR = '\x19'
-# These are non-printable chars, so they should never appear in the input,
-# I guess. But maybe we can find better chars that are even less risky.
-format_chars = '\x0E\x0F\x10\x11\x12\x13\x14\x15\x16\x17\x18'
+from poezio.ui.consts import FORMAT_CHAR
 
-class DummyWin(object):
-    def __getattribute__(self, name):
+if TYPE_CHECKING:
+    from _curses import _CursesWindow  # pylint: disable=E0611
+
+
+class DummyWin:
+    def __getattribute__(self, name: str):
         if name != '__bool__':
             return lambda *args, **kwargs: (0, 0)
         else:
             return object.__getattribute__(self, name)
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         return False
 
 
-class Win(object):
-    def __init__(self):
-        self._win = None
+class Win:
+    __slots__ = ('_win', 'height', 'width', 'y', 'x')
+
+    def __init__(self) -> None:
+        self._win = None  # type: _CursesWindow
         self.height, self.width = 0, 0
 
-    def _resize(self, height, width, y, x):
+    def _resize(self, height: int, width: int, y: int, x: int) -> None:
         if height == 0 or width == 0:
             self.height, self.width = height, width
             return
@@ -50,16 +56,16 @@ class Win(object):
             if self._win is None:
                 self._win = DummyWin()
 
-    def resize(self, height, width, y, x):
+    def resize(self, height: int, width: int, y: int, x: int) -> None:
         """
         Override if something has to be done on resize
         """
         self._resize(height, width, y, x)
 
-    def _refresh(self):
+    def _refresh(self) -> None:
         self._win.noutrefresh()
 
-    def addnstr(self, *args):
+    def addnstr(self, *args) -> None:
         """
         Safe call to addnstr
         """
@@ -71,7 +77,23 @@ class Win(object):
             # of the screen.
             pass
 
-    def addstr(self, *args):
+    @contextmanager
+    def colored_text(self, color: Optional[Tuple]=None, attr: Optional[int]=None):
+        """Context manager which sets up an attr/color when inside"""
+        if color is None and attr is None:
+            yield None
+            return
+        if color is not None:
+            mode = to_curses_attr(color)
+            if attr is not None:
+                mode = mode | attr
+        else:
+            mode = attr
+        self._win.attron(mode)
+        yield None
+        self._win.attroff(mode)
+
+    def addstr(self, *args) -> None:
         """
         Safe call to addstr
         """
@@ -80,13 +102,13 @@ class Win(object):
         except:
             pass
 
-    def move(self, y, x):
+    def move(self, y: int, x: int) -> None:
         try:
             self._win.move(y, x)
         except:
             pass
 
-    def addstr_colored(self, text, y=None, x=None):
+    def addstr_colored(self, text: str, y: Optional[int] = None, x: Optional[int] = None) -> None:
         """
         Write a string on the window, setting the
         attributes as they are in the string.
@@ -95,14 +117,16 @@ class Win(object):
         \x191}Bonj\x192}our → 'Bonj' in red and 'our' in green
         next_attr_char is the \x19 delimiter
         attr_char is the char following it, it can be
-        one of 'u', 'b', 'c[0-9]'
+        one of 'u', 'b', 'i', 'c[0-9]'
         """
         if y is not None and x is not None:
             self.move(y, x)
         next_attr_char = text.find(FORMAT_CHAR)
+        attr_italic = curses.A_ITALIC if hasattr(
+            curses, 'A_ITALIC') else curses.A_REVERSE
         while next_attr_char != -1 and text:
             if next_attr_char + 1 < len(text):
-                attr_char = text[next_attr_char+1].lower()
+                attr_char = text[next_attr_char + 1].lower()
             else:
                 attr_char = str()
             if next_attr_char != 0:
@@ -113,8 +137,12 @@ class Win(object):
                 self._win.attron(curses.A_UNDERLINE)
             elif attr_char == 'b':
                 self._win.attron(curses.A_BOLD)
-            if (attr_char in string.digits or attr_char == '-') and attr_char != '':
-                color_str = text[next_attr_char+1:text.find('}', next_attr_char)]
+            elif attr_char == 'i':
+                self._win.attron(attr_italic)
+            if (attr_char in string.digits
+                    or attr_char == '-') and attr_char != '':
+                color_str = text[next_attr_char +
+                                 1:text.find('}', next_attr_char)]
                 if ',' in color_str:
                     tup, char = read_tuple(color_str)
                     self._win.attron(to_curses_attr(tup))
@@ -125,25 +153,27 @@ class Win(object):
                             self._win.attron(curses.A_UNDERLINE)
                         elif char == 'b':
                             self._win.attron(curses.A_BOLD)
+                        elif char == 'i':
+                            self._win.attron(attr_italic)
                     else:
                         # this will reset previous bold/uderline sequences if any was used
                         self._win.attroff(curses.A_UNDERLINE)
                         self._win.attroff(curses.A_BOLD)
                 elif color_str:
                     self._win.attron(to_curses_attr((int(color_str), -1)))
-                text = text[next_attr_char+len(color_str)+2:]
+                text = text[next_attr_char + len(color_str) + 2:]
             else:
-                text = text[next_attr_char+2:]
+                text = text[next_attr_char + 2:]
             next_attr_char = text.find(FORMAT_CHAR)
         self.addstr(text)
 
-    def finish_line(self, color=None):
+    def finish_line(self, color: Optional[Tuple] = None) -> None:
         """
         Write colored spaces until the end of line
         """
         (y, x) = self._win.getyx()
         size = self.width - x
         if color:
-            self.addnstr(' '*size, size, to_curses_attr(color))
+            self.addnstr(' ' * size, size, to_curses_attr(color))
         else:
-            self.addnstr(' '*size, size)
+            self.addnstr(' ' * size, size)
