@@ -6,9 +6,11 @@ from poezio.plugin import BasePlugin
 from slixmpp.xmlstream.matcher import StanzaPath
 from slixmpp.xmlstream.handler import Callback
 
+
 class Plugin(BasePlugin):
     def init(self):
-        self.core.xmpp.register_handler(Callback('Iq_show', StanzaPath('iq'), self.handle_iq))
+        self.core.xmpp.register_handler(
+            Callback('Iq_show', StanzaPath('iq'), self.handle_iq))
 
     def handle_iq(self, iq):
         self.api.information('%s' % iq, 'Iq')
