@@ -1,92 +1,108 @@
 """
 Module containing various decorators
 """
+from typing import Any, Callable, List, Optional
 
 from poezio import common
 
-class RefreshWrapper(object):
+
+class RefreshWrapper:
     def __init__(self):
         self.core = None
 
-    def conditional(self, func):
+    def conditional(self, func: Callable) -> Callable:
         """
         Decorator to refresh the UI if the wrapped function
         returns True
         """
+
         def wrap(*args, **kwargs):
             ret = func(*args, **kwargs)
             if self.core and ret:
                 self.core.refresh_window()
             return ret
+
         return wrap
 
-    def always(self, func):
+    def always(self, func: Callable) -> Callable:
         """
         Decorator that refreshs the UI no matter what after the function
         """
+
         def wrap(*args, **kwargs):
             ret = func(*args, **kwargs)
             if self.core:
                 self.core.refresh_window()
             return ret
+
         return wrap
 
-    def update(self, func):
+    def update(self, func: Callable) -> Callable:
         """
         Decorator that only updates the screen
         """
+
         def wrap(*args, **kwargs):
             ret = func(*args, **kwargs)
             if self.core:
                 self.core.doupdate()
             return ret
+
         return wrap
 
+
 refresh_wrapper = RefreshWrapper()
 
-class CommandArgParser(object):
+
+class CommandArgParser:
     """Modify the string argument of the function into a list of strings
     containing the right number of extracted arguments, or None if we don’t
     have enough.
     """
+
     @staticmethod
-    def raw(func):
+    def raw(func: Callable) -> Callable:
         """Just call the function with a single string, which is the original string
         untouched
         """
+
         def wrap(self, args, *a, **kw):
             return func(self, args, *a, **kw)
+
         return wrap
 
     @staticmethod
-    def ignored(func):
+    def ignored(func: Callable) -> Callable:
         """
         Call the function without any argument
         """
-        def wrap(self, args, *a, **kw):
+
+        def wrap(self, args=None, *a, **kw):
             return func(self, *a, **kw)
+
         return wrap
 
     @staticmethod
-    def quoted(mandatory, optional=0, defaults=None,
+    def quoted(mandatory: int,
+               optional=0,
+               defaults: Optional[List[Any]] = None,
                ignore_trailing_arguments=False):
-
         """The function receives a list with a number of arguments that is between
         the numbers `mandatory` and `optional`.
 
         If the string doesn’t contain at least `mandatory` arguments, we return
-        None because the given arguments  are invalid.
+        None because the given arguments are invalid.
 
         If there are any remaining arguments after `mandatory` and `optional`
         arguments have been found (and “ignore_trailing_arguments" is not True),
-        we happen them to the last argument of the list.
+        we append them to the last argument of the list.
 
-        An argument is a string (with or without whitespaces) between to quotes
+        An argument is a string (with or without whitespaces) between two quotes
         ("), or a whitespace separated word (if not inside quotes).
 
         The argument `defaults` is a list of strings that are used when an
         optional argument is missing.  For example if we accept one optional
-        argument, zero is available but we have one value in the `defaults`
+        argument and none is provided, but we have one value in the `defaults`
         list, we use that string inplace. The `defaults` list can only
         replace missing optional arguments, not mandatory ones. And it
         should not contain more than `mandatory` values. Also you cannot
@@ -113,32 +129,48 @@ class CommandArgParser(object):
         ['un et demi', 'deux', 'trois quatre cinq six']
 
         """
-        if defaults is None:
-            defaults = []
-        def first(func):
-            def second(self, args, *a, **kw):
-                default_args = defaults
+        default_args_outer = defaults or []
+
+        def first(func: Callable):
+            def second(self, args: str, *a, **kw):
+                default_args = default_args_outer
                 if args and args.strip():
-                    args = common.shell_split(args)
+                    split_args = common.shell_split(args)
                 else:
-                    args = []
-                if len(args) < mandatory:
+                    split_args = []
+                if len(split_args) < mandatory:
                     return func(self, None, *a, **kw)
-                res, args = args[:mandatory], args[mandatory:]
+                res, split_args = split_args[:mandatory], split_args[
+                    mandatory:]
                 if optional == -1:
-                    opt_args = args[:]
+                    opt_args = split_args[:]
                 else:
-                    opt_args = args[:optional]
+                    opt_args = split_args[:optional]
 
                 if opt_args:
                     res += opt_args
-                    args = args[len(opt_args):]
+                    split_args = split_args[len(opt_args):]
                     default_args = default_args[len(opt_args):]
                 res += default_args
-                if args and res and not ignore_trailing_arguments:
-                    res[-1] += " " + " ".join(args)
+                if split_args and res and not ignore_trailing_arguments:
+                    res[-1] += " " + " ".join(split_args)
                 return func(self, res, *a, **kw)
+
             return second
+
         return first
 
+
 command_args_parser = CommandArgParser()
+
+
+def deny_anonymous(func: Callable) -> Callable:
+    """Decorator to disable commands when using an anonymous account."""
+    def wrap(self: 'RosterInfoTab', *args, **kwargs):
+        if self.core.xmpp.anon:
+            return self.core.information(
+                'This command is not available for anonymous accounts.',
+                'Info'
+            )
+        return func(self, *args, **kwargs)
+    return wrap
