@@ -1,7 +1,7 @@
 """
 This plugin broadcasts a message to all your joined rooms.
 
-.. note:: With great power comes great responsability.
+.. note:: With great power comes great responsibility.
           Use with moderation.
 
 Command
@@ -16,15 +16,18 @@ Command
 
 
 """
-from plugin import BasePlugin
-from tabs import MucTab
+from poezio.plugin import BasePlugin
+from poezio.tabs import MucTab
+
 
 class Plugin(BasePlugin):
     def init(self):
-        self.api.add_command('amsg', self.command_amsg,
-                usage='<message>',
-                short='Broadcast a message',
-                help='Broadcast the message to all the joined rooms.')
+        self.api.add_command(
+            'amsg',
+            self.command_amsg,
+            usage='<message>',
+            short='Broadcast a message',
+            help='Broadcast the message to all the joined rooms.')
 
     def command_amsg(self, args):
         for room in self.core.tabs:
